/*
 * CLI-commands.c
 *
 *  Created on: 24 oct. 2017
 *      Author: Carlos
 */

/* FreeRTOS includes. */
#include "FreeRTOS.h"
#include "task.h"

/* Standard includes. */
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "des.h"
#include "3des.h"
#include "md5.h"
#include <time.h>
#include "aes_soft.h"
#include "rsa.h"
#include <Math.h>

/* FreeRTOS+CLI includes. */
#include "FreeRTOS_CLI.h"

//Variables necesarias para la gestion de los mensajes y la encriptacion
 uint8_t *datos = NULL, *clave = NULL, *vector = NULL, *modo = NULL;
int tam_datos = 0;
unsigned int tam_datos_sin_pad = 0;
int size = 0;
int algoritmo = 0;
int tam_block = 8;
int tam_clave = 8;

//Guarda la Clave Publica y privada de RSA
struct public_key_class pub[1];
struct private_key_class pri[1];

//Si login = 1 esta logeado en caso contrario no
int login = 0;

/*
 *La funcin que registra los comandos que están definidos dentro de este archivo.
 */
void vRegisterCLICommands(void);

int numerodigitos(long num);

/*
 * Transforma la cadena de Numeros a Caracteres en Hexadecimal
 */
void toStr(uint8_t* hex, unsigned char* str, int len);

/*
 * Transforma Cadenas con caracteres en Hexadecimal en Cadenas uint8_t;
 */
void toUint8(uint8_t* hex, unsigned char* str, int len);

/*
 * Hace xor sobre los parametros que le pasamos.
 */
void xor(uint8_t* str, uint8_t* str2);

//Funcin que se encarga de inicializar los vectores necesarios para la encriptacin
static BaseType_t prvInicializa(char *pcWriteBuffer, size_t xWriteBufferLen,
                                const char *pcCommandString);

//Funcin encargada de aadir los datos recibidos a Vector de datos
static BaseType_t prvTextDataCommand(char *pcWriteBuffer,
                                     size_t xWriteBufferLen,
                                     const char *pcCommandString);

//Muestra el vector datos en Ascii
static BaseType_t prvDatosCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                  const char *pcCommandString);

//Muestra el vector datos en Hexadecimal
static BaseType_t prvDatosHexCommand(char *pcWriteBuffer,
                                     size_t xWriteBufferLen,
                                     const char *pcCommandString);

//Hace un XOR de los parametros recibidos
static BaseType_t prvXORCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                const char *pcCommandString);

//Compara el Hash recibido con el calculado de los datos
static BaseType_t prvMD5Command(char *pcWriteBuffer, size_t xWriteBufferLen,
                                const char *pcCommandString);

//Genera una clave Aleotoria
static BaseType_t prvGenKeyCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                   const char *pcCommandString);

//Encripta o desencripta en DES
static BaseType_t prvDESCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                const char *pcCommandString);

//Encripta o desencripta en 3DES
static BaseType_t prvTDESCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                 const char *pcCommandString);

//Encripta o desencripta en AES Software
static BaseType_t prvAESCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                const char *pcCommandString);

//Encripta o desencripta en AES Hardaware
static BaseType_t prvAES_HARDCommand(char *pcWriteBuffer,
                                     size_t xWriteBufferLen,
                                     const char *pcCommandString);

//Encripta o desencripta en RSA
static BaseType_t prvRSACommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                const char *pcCommandString);

//Devuelve la clave Publica
static BaseType_t prvGetPublicKeyCommand(char *pcWriteBuffer,
                                         size_t xWriteBufferLen,
                                         const char *pcCommandString);
//Recibe la clave Publica
static BaseType_t prvSetPublicKeyCommand(char *pcWriteBuffer,
                                         size_t xWriteBufferLen,
                                         const char *pcCommandString);

//Obtiene el mensaje RSA
static BaseType_t prvTextRSACommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                    const char *pcCommandString);

//inicia Los Bufferes RSA
static BaseType_t prvIniRSACommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                   const char *pcCommandString);

//Obtiene la contrasea cifrada y cambia el color de los led en caso de que sea correcto
static BaseType_t prvLOGINCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                  const char *pcCommandString);

//Si esta logeado cambia los led a rojo
static BaseType_t prvLOGOUTCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                   const char *pcCommandString);
// Devuelve el checksum de los datos
void prvMD5(char *checksum);

/* Estructura que define the "genKey" comando de de linea de comando.
 * Genera una clave de encriptacion. */
static const CLI_Command_Definition_t genKey = {
        "keygen", /* The command string to type. */
        "\r\ngenKey:\r\n Genera una clave de encriptacion\r\n",
        prvGenKeyCommand, /* The function to run. */
        1 /* No parameters are expected. */
};

/* Estructura que define the DES comando de de linea de comando.
 * Encripta en DES. */
static const CLI_Command_Definition_t DES = {
        "DES", /* The command string to type. */
        "\r\nDES:\r\n Encripta o desencripta \r\n", prvDESCommand, /* The function to run. */
        1 };

static const CLI_Command_Definition_t Inicializa = {
        "ini", /* The command string to type. */
        "\r\nini:\r\n incializa los vectores \r\n", prvInicializa, /* The function to run. */
        -1 /* No parameters are expected. */
};

static const CLI_Command_Definition_t text_data = {
        "text_data", /* The command string to type. */
        "\r\ntext_data:\r\n Obtiene los datos del bloque\r\n",
        prvTextDataCommand, /* The function to run. */
        1 /* No parameters are expected. */
};

static const CLI_Command_Definition_t valid_md5 = {
        "MD5", /* The command string to type. */
        "\r\nMD5:\r\n Valida los datos recibidos\r\n", prvMD5Command, /* The function to run. */
        1 /* No parameters are expected. */
};

static const CLI_Command_Definition_t muestraDatos = {
        "Datos", /* The command string to type. */
        "\r\nDatos:\r\n Devuelve los datos\r\n", prvDatosCommand, /* The function to run. */
        -1 /* No parameters are expected. */
};

static const CLI_Command_Definition_t xorcommand = {
        "xor", /* The command string to type. */
        "\r\nxor:\r\nXor de los parametros\r\n", prvXORCommand, /* The function to run. */
        2 /* No parameters are expected. */
};

static const CLI_Command_Definition_t datos_hex = {
        "datos_hex", "\r\ndatos_hex:\r\nDevuelven los datos en Hexadecimal\r\n",
        prvDatosHexCommand, 0 };

static const CLI_Command_Definition_t AES = {
        "AES",
        "\r\nAES:\r\nRealiza la encriptacin decriptacin con AES Software\r\n",
        prvAESCommand, 1 };

static const CLI_Command_Definition_t AES_Hard =
        { "AES_H",
          "\r\nAES_H:\r\nRealiza la encriptacin decriptacin con AES Hardware\r\n",
          prvAES_HARDCommand, 1 };

static const CLI_Command_Definition_t TDES = {
        "TDES",
        "\r\nTDES_H:\r\nRealiza la encriptacin decriptacin con TDES\r\n",
        prvTDESCommand, 1 };

static const CLI_Command_Definition_t RSA = {
        "RSA", "\r\nRSA:\r\nRealiza la encriptacin decriptacin RSA\r\n",
        prvRSACommand, 1 };

static const CLI_Command_Definition_t getPublicKey = {
        "getPublicKey",
        "\r\ngetPublicKey:\r\nObtiene la clave publica y su modulo\r\n",
        prvGetPublicKeyCommand, 0 };

static const CLI_Command_Definition_t setPublicKey =
        { "setPublicKey",
          "\r\nsetPublicKey:\r\nPone en el sistema los parametros de clave publico y modulo \r\n",
          prvSetPublicKeyCommand, 2 };

static const CLI_Command_Definition_t dataRSA = {
        "rsa_data", "\r\nrsa_data:\r\nEnvia los datos RSA \r\n",
        prvTextRSACommand, 1 };

static const CLI_Command_Definition_t iniRSA = {
        "IniRSA", "\r\nIniRSA:\r\nEnvia tamao de los datos RSA \r\n",
        prvIniRSACommand, 2 };

static const CLI_Command_Definition_t LOGIN = {
        "LOGIN", "\r\nLOGIN:\r\nLOGIN EN EL SISTEMA\r\n", prvLOGINCommand, 1 };
static const CLI_Command_Definition_t LOGOUT = { "LOGOUT",
                                                 "\r\nLOGOUT:\r\nLOGOUT\r\n",
                                                 prvLOGOUTCommand, 0 };
void vRegisterCLICommands(void)
{
    /* Registra todos los comandos definidos anteriormente. */
    FreeRTOS_CLIRegisterCommand(&genKey);
    FreeRTOS_CLIRegisterCommand(&DES);
    FreeRTOS_CLIRegisterCommand(&Inicializa);
    FreeRTOS_CLIRegisterCommand(&text_data);
    FreeRTOS_CLIRegisterCommand(&valid_md5);
    FreeRTOS_CLIRegisterCommand(&muestraDatos);
    FreeRTOS_CLIRegisterCommand(&xorcommand);
    FreeRTOS_CLIRegisterCommand(&datos_hex);
    FreeRTOS_CLIRegisterCommand(&AES);
    FreeRTOS_CLIRegisterCommand(&AES_Hard);
    FreeRTOS_CLIRegisterCommand(&TDES);
    FreeRTOS_CLIRegisterCommand(&RSA);
    FreeRTOS_CLIRegisterCommand(&getPublicKey);
    FreeRTOS_CLIRegisterCommand(&setPublicKey);
    FreeRTOS_CLIRegisterCommand(&dataRSA);
    FreeRTOS_CLIRegisterCommand(&iniRSA);
    FreeRTOS_CLIRegisterCommand(&LOGIN);
    FreeRTOS_CLIRegisterCommand(&LOGOUT);
}

/*
 * Iniciamos los vectores con las parametros recibidos
 */

static BaseType_t prvInicializa(char *pcWriteBuffer, size_t xWriteBufferLen,
                                const char *pcCommandString)
{
    unsigned char *prmAlgoritmo, *prmTamDatos, *prmModo, *prmClave, *prmVector;

    BaseType_t xAlgoritmoLength, xTamDatosLength, xModoLength, xClaveLength,
            xVectorLength;

    (void) pcCommandString;
    (void) xWriteBufferLen;
    configASSERT(pcWriteBuffer);

    prmAlgoritmo = (unsigned char *) FreeRTOS_CLIGetParameter(
            pcCommandString, 1, &xAlgoritmoLength);
    prmTamDatos = (unsigned char *) FreeRTOS_CLIGetParameter(pcCommandString, 2,
                                                             &xTamDatosLength);
    prmModo = (unsigned char *) FreeRTOS_CLIGetParameter(pcCommandString, 3,
                                                         &xModoLength);
    prmClave = (unsigned char *) FreeRTOS_CLIGetParameter(pcCommandString, 4,
                                                          &xClaveLength);
    prmVector = (unsigned char *) FreeRTOS_CLIGetParameter(pcCommandString, 5,
                                                           &xVectorLength);

    prmAlgoritmo[xAlgoritmoLength] = 0x00;
    prmTamDatos[xTamDatosLength] = 0x00;
    prmModo[xModoLength] = 0x00;
    prmClave[xClaveLength] = 0x00;
    prmVector[xVectorLength] = 0x00;

    size = 0;

    //Liberamos el espacio si es distinto de Null
    if (vector != NULL)
    {
        //vector = NULL;
        vPortFree(vector);
    }

//memset(vector, 0x00, 8);

//Liberamos el espacio si es distinto de Null
    if (modo != NULL)
    {
        //modo = NULL;
        vPortFree(modo);
    }

//memset(modo, 0x00, 8);

//Liberamos el espacio si es distinto de Null
    if (clave != NULL)
    {
        //clave = NULL;
        vPortFree(clave);

    }

//memset(clave, 0x00, 8);

//Liberamos el espacio si es distinto de Null
    if (datos != NULL)
    {
        //datos = NULL;
        vPortFree(datos);
    }
    tam_datos_sin_pad = atoi((char *) prmTamDatos);

    if (tam_datos_sin_pad != 0)
    {

        algoritmo = atoi((char *) prmAlgoritmo);

        //Segn el algoritmo ponemos el tamao de bloque y la clave.
        if (algoritmo == 0)
        {
            tam_block = 8;
            tam_clave = 8;
        }
        else if (algoritmo == 1)
        {
            tam_block = 8;
            tam_clave = 24;
        }
        else if (algoritmo == 2)
        {
            tam_block = 16;
            if (xClaveLength / 2 == 16)
            {
                tam_clave = 16;
            }
            else if (xClaveLength / 2 == 24)
            {
                tam_clave = 24;
            }
            else if (xClaveLength / 2 == 32)
            {
                tam_clave = 32;
            }
            else
            {
                tam_clave = 0;
            }
        }
        else if (algoritmo == 3)
        {
            tam_block = 16;
            if (xClaveLength / 2 == 16)
            {
                tam_clave = 16;
            }
            else if (xClaveLength / 2 == 24)
            {
                tam_clave = 24;
            }
            else if (xClaveLength / 2 == 32)
            {
                tam_clave = 32;
            }
            else
            {
                tam_clave = 0;
            }
        }

        if (tam_clave == (xClaveLength / 2))
        {
            if ((strcmp((char *) prmModo, "ECB") == 0)
                    || (((strcmp((char *) prmModo, "CBC") == 0)
                            || (strcmp((char *) prmModo, "CFB") == 0)
                            || (strcmp((char *) prmModo, "PCBC") == 0)
                            || (strcmp((char *) prmModo, "OFB") == 0))
                            && (tam_block == (xVectorLength / 2))))

            {

                //Obtenemos el tamao de los datos que se van a recibir.
                if ((tam_datos_sin_pad % tam_block) > 0)
                {
                    tam_datos = tam_datos_sin_pad
                            + (tam_block - (tam_datos_sin_pad % tam_block));
                }
                else
                {
                    tam_datos = tam_datos_sin_pad;
                }

                //Reservamos la memoria necesaria para los datos que vamos a recibir
                datos = (uint8_t*) pvPortMalloc(
                        (tam_datos + 1) * sizeof(uint8_t));
                memset(datos, 0, tam_datos);

                //Reservamos la memoria necesaria para los datos que vamos a recibir
                clave = (uint8_t*) pvPortMalloc((tam_clave) * sizeof(uint8_t));

                modo = (uint8_t*) pvPortMalloc(5 * sizeof(uint8_t));

                unsigned int i = 0;
                //Convertimos a Ascii los datos recibidos
                for (i = 0; i < tam_clave; i++)
                {
                    sscanf((char *) prmClave + 2 * i, "%02x", &clave[i]);

                }
                if (strcmp((char *) prmModo, "ECB") != 0)
                {
                    vector = (uint8_t*) pvPortMalloc(
                            (tam_block + 1) * sizeof(uint8_t));
                    for (i = 0; i < 2 * tam_block; i++)
                    {
                        sscanf((char *) prmVector + 2 * i, "%02x", &vector[i]);
                    }
                    vector[tam_block] = 0x00;
                }
                else
                {
                    vector = (uint8_t*) pvPortMalloc(sizeof(uint8_t));
                    //vector= NULL;
                }

                //strncpy((char *) modo, (char *) prmModo, xModoLength);

                datos[tam_datos] = 0x00;

                /*if (strcmp((char *) modo, "PCBC") == 0)
                {
                    strncpy((char *) modo, (char *) prmModo, 4);
                    modo[4] = 0x00;
                }
                else
                {*/
                strncpy((char *) modo, (char *) prmModo, 3);
                    modo[3] = 0x00;
                //}
                sprintf(pcWriteBuffer,
                        "Vector %s - Modo %s - clave %s %d %d %d-OK\n", vector,
                        modo, clave, algoritmo, tam_datos, tam_clave);
            }
            else
            {
                sprintf(pcWriteBuffer, "Error con el modo -KO\n");
            }

        }
        else
        {
            sprintf(pcWriteBuffer, "El tamao de la clave es erroneo -KO\n");
        }

    }
    else
    {
        sprintf(pcWriteBuffer, "ERROR AL CREAR EL BUFFER DE DATOS-KO\n");
    }

    return pdFALSE;
}

/*
 * Aade al vector datos el vector recibido despues de transformalo de Hexadecimal a Ascii
 */

static BaseType_t prvTextDataCommand(char *pcWriteBuffer,
                                     size_t xWriteBufferLen,
                                     const char *pcCommandString)
{
    unsigned char *aux;
    BaseType_t xParameter1StringLength;

    aux = (unsigned char *) FreeRTOS_CLIGetParameter(pcCommandString, 1,
                                                     &xParameter1StringLength);
    aux[xParameter1StringLength] = 0x00;

    //El tamao de los datos aadidos sera la mitad de los recibidos
    uint8_t len = xParameter1StringLength / 2;

    //Si el tamao que queda de buffer es igual o mayor que el tamao del parametro lo aadimos.
    if (size + len <= tam_datos)
    {
        //Si datos esta inicializado aadimos el parametro recibido
        if (datos != NULL)
        {

            uint8_t* bytearray = (uint8_t*) pvPortMalloc(
                    (len + 2) * sizeof(uint8_t));

            //transformamos el de hexadecimal a uint8_t
            toUint8(aux, bytearray, xParameter1StringLength);

            //Copiamos los aux al vector de datos
            memcpy(&datos[size], bytearray, len * sizeof(uint8_t));
            //memcpy(datos+size, bytearray, len * sizeof(uint8_t));

            // Sumamos el tamao aadido
            size = size + len;
            sprintf(pcWriteBuffer, "%d - %d -OK", tam_datos, size);
            //datos[strlen(datos)] = 0x00;
            vPortFree(bytearray);

        }
        else
        {   //Devolmemos el error
            sprintf(pcWriteBuffer, "ERROR - Datos no iniciados -KO");
        }
    }
    else
    {
        sprintf(pcWriteBuffer,
                "ERROR - Los datos son mayores que el buffer -KO");
    }

    return pdFALSE;

}

/*
 * Devuelve los datos en ASCII
 */
static BaseType_t prvDatosCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                  const char *pcCommandString)
{
    unsigned char *chk;

    BaseType_t xChkLength;

    chk = (unsigned char *) FreeRTOS_CLIGetParameter(pcCommandString, 1,
                                                     &xChkLength);

    chk[xChkLength] = 0x00;

    static BaseType_t bloque = -1;
    int tam_bloque = 150;
    //Si no hay datos se devuelve error.
    if ((tam_datos == 0) || (tam_datos == NULL))
    {
        sprintf(pcWriteBuffer, "Error -KO");

        return pdFALSE;
    }
    else
    {

        int numbloques = tam_datos / tam_bloque
                + ((tam_datos % tam_bloque) ? 1 : 0);

        if (bloque == -1)
        { //Mandamos la etiqueta antes de enviar los datos

            if (strcmp((char *) chk, "1") == 0)
            {
                sprintf(pcWriteBuffer, "Mensaje: ");
            }
            else
            {
                sprintf(pcWriteBuffer, "Datos: ");
            }
            bloque = bloque + 1;
            return pdTRUE;
        }
        else if (bloque == numbloques)
        {
            //DEvolvemos el checksum de los datos para comprobar que se han enviado correctamente
            unsigned char umd5[32];
            prvMD5((char *) umd5);
            sprintf(pcWriteBuffer, "\r\nchecksum:%s-OK\r\n", umd5);
            bloque = -1;
            return pdFALSE;
        }

        else if (bloque == numbloques - 1)
        {
            int t = tam_datos - (tam_bloque * bloque);
            uint8_t* str = (uint8_t*) pvPortMalloc(t * sizeof(uint8_t));

            memcpy(str, &datos[bloque * tam_bloque], t * sizeof(uint8_t));
            //str[t] = 0x00;
            if (strcmp((char *) chk, "1") == 0)
            {
                sprintf(pcWriteBuffer, "%s", str);
                bloque = bloque + 1;
                //Devolvemos pdFALSE puesto que ya hemos terminado
                vPortFree(str);
                return pdTRUE;
            }
            else
            {
                sprintf(pcWriteBuffer, "%s-OK", str);
                bloque = -1;
                //Devolvemos pdFALSE puesto que ya hemos terminado
                vPortFree(str);
                return pdFALSE;
            }

        }
        else
        {

            uint8_t* str = (uint8_t*) pvPortMalloc(
                    tam_bloque * sizeof(uint8_t));
            memcpy(str, &datos[bloque * tam_bloque],
                   tam_bloque * sizeof(uint8_t));
            //str[tam_bloque] = 0x00;
            sprintf(pcWriteBuffer, "%s", str);
            bloque = bloque + 1;
            vPortFree(str);

            //Devolvemos pdTRUE para enviar el prximo bloque
            return pdTRUE;

        }
    }
}

/*
 * Devuelve los datos en Hexadecimal
 */

static BaseType_t prvDatosHexCommand(char *pcWriteBuffer,
                                     size_t xWriteBufferLen,
                                     const char *pcCommandString)
{

    static BaseType_t bloque = -1;
    int tam_bloque = 150;

    int numbloques = tam_datos / tam_bloque
            + ((tam_datos % tam_bloque) ? 1 : 0);

    if (bloque == -1)
    {
        sprintf(pcWriteBuffer, "Mensaje: ");
        bloque = bloque + 1;
        return pdTRUE;
    }
    else if (bloque == numbloques)
    {
        unsigned char umd5[32];
        prvMD5((char *) umd5);
        sprintf(pcWriteBuffer, "\r\nchecksum:%s-OK\r\n", umd5);
        bloque = -1;
        return pdFALSE;
    }

    else if (bloque == numbloques - 1)
    {
        int t = tam_datos - (tam_bloque * bloque);
        uint8_t* str = (uint8_t*) pvPortMalloc(t * sizeof(uint8_t));
        uint8_t* hex = (uint8_t*) pvPortMalloc(
                2 * tam_bloque * sizeof(uint8_t));
        memcpy(str, &datos[bloque * tam_bloque], t * sizeof(uint8_t));
        //toStr(pcWriteBuffer, str, tam_bloque);
        int i = 0;
        for (i = 0; i < t; i++)
        {
            sprintf((char *) hex + i * 2, "%02X", str[i]);
        }
        sprintf(pcWriteBuffer, "%s", hex);
        vPortFree(str);
        vPortFree(hex);
        bloque = bloque + 1;
        return pdTRUE;
    }
    else
    {
        uint8_t* str = (uint8_t*) pvPortMalloc(tam_bloque * sizeof(uint8_t));
        uint8_t* hex = (uint8_t*) pvPortMalloc(
                2 * tam_bloque * sizeof(uint8_t));
        memcpy(str, &datos[bloque * tam_bloque], tam_bloque * sizeof(uint8_t));
        //toStr(pcWriteBuffer, str, tam_bloque);
        int i = 0;
        for (i = 0; i < tam_bloque; i++)
        {
            sprintf((char *) hex + i * 2, "%02X", str[i]);
        }
        sprintf(pcWriteBuffer, "%s", hex);
        vPortFree(str);
        vPortFree(hex);
        bloque = bloque + 1;
        return pdTRUE;

    }

}

/*
 * Encripta o desencripta usando el Algoritmo DES
 */

static BaseType_t prvDESCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                const char *pcCommandString)
{

    //Obtenemos el parametro que nos indica si encriptar o desencriptar.
    unsigned char *encriptar;
    BaseType_t encriptarStringLength;
    encriptar = (unsigned char *) FreeRTOS_CLIGetParameter(
            pcCommandString, 1, &encriptarStringLength);
    encriptar[encriptarStringLength] = 0x00;

    TickType_t xTimeBefore, xTotalTime;
    xTimeBefore = xTaskGetTickCount();

    //Obtenemos el numero de bloques
    int numbloques = tam_datos / tam_block + ((tam_datos % tam_block) ? 1 : 0);

    //Reservamos memoria para las variables que usamos para encriptar y desencriptar
    uint8_t *block = (uint8_t*) pvPortMalloc(tam_block * sizeof(uint8_t));
    uint8_t *block_aux = (uint8_t*) pvPortMalloc(tam_block * sizeof(uint8_t));
    uint8_t *block_process = (uint8_t*) pvPortMalloc(
            tam_block * sizeof(uint8_t));
    uint8_t* resultado = (uint8_t*) pvPortMalloc(
            numbloques * tam_block * sizeof(uint8_t));

    MAP_WDT_A_holdTimer();
    //Preparamos la clave
    key_set* key_sets = (key_set*) pvPortMalloc(17 * sizeof(key_set));
    memset(key_sets, 0x00, 17 * sizeof(key_set));
    generate_sub_keys(clave, key_sets);

    //Hacemos una copia del vector
    uint8_t* init_vector = (unsigned char*) pvPortMalloc(
            tam_clave * sizeof(uint8_t));
    memcpy(init_vector, vector, tam_clave * sizeof(uint8_t));

    int i = 0;

    //sprintf(pcWriteBuffer, "Datos - %s - %d Clave - %s - %d k %s c %s d %s -\r\n", datos, tam_datos, clave, tam_clave, key_sets->k, key_sets->c, key_sets->d);

    if (strcmp((char *) modo, "ECB") == 0)
    {   //Modo ECB

        if (strcmp((char *) encriptar, "1") == 0)
        {   //Encriptamos

            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                //Encriptamos el bloque
                process_message(block, block_process, key_sets, 1);

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));
            }
        }
        else
        {   //Desencriptamos

            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                process_message(block, block_process, key_sets, 0);

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));
            }
        }
    }
    else if (strcmp((char *) modo, "CBC") == 0)
    {   //MODO CBC
        if (strcmp((char *) encriptar, "1") == 0)
        {   //Encriptamos

            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                xor(block, init_vector);

                //Encriptamos el bloque
                process_message(block, block_process, key_sets, 1);
                memcpy(init_vector, block_process, tam_clave * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));
            }
        }
        else
        {   //Desencriptamos

            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));

                //Desencriptamos el bloque
                process_message(block, block_process, key_sets, 0);
                xor(block_process, init_vector);
                memcpy(init_vector, block, tam_clave * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));
            }
        }

    }
    else if (strcmp((char *) modo, "PCB") == 0)
    { //MODO PCBC
        if (strcmp((char *) encriptar, "1") == 0)
        {   //Encriptamos

            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                memcpy(block_aux, block, tam_block * sizeof(uint8_t));
                xor(block, init_vector);

                //Encriptamos el bloque
                process_message(block, block_process, key_sets, 1);
                xor(block_aux, block_process);
                memcpy(init_vector, block_aux, tam_block * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));

            }
        }
        else
        {   //Desencriptamos

            for (i = 0; i < numbloques; i++)
            {   //Encriptamos

                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                memcpy(block_aux, block, tam_block * sizeof(uint8_t));

                //Desencriptamos el bloque
                process_message(block, block_process, key_sets, 0);
                xor(block_process, init_vector);
                xor(block_aux, block_process);
                memcpy(init_vector, block_aux, tam_block * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));

            }
        }
    }
    else if (strcmp((char *) modo, "OFB") == 0)
    {   //MODO OFB
        //En este modo da igual encriptar o desencriptar
        for (i = 0; i < numbloques; i++)
        {
            //Dividimos en bloques los datos
            memcpy(block, &datos[i * tam_block], tam_block * sizeof(uint8_t));

            //Encriptamos el bloque
            process_message(init_vector, block_process, key_sets, 1);
            memcpy(init_vector, block_process, tam_block * sizeof(uint8_t));
            xor(block_process, block);

            //Copiamos el bloque en resultado
            memcpy(resultado + tam_block * i, block_process,
                   tam_block * sizeof(uint8_t));
        }

    }
    else if (strcmp((char *) modo, "CFB") == 0)
    {   //MODO CFB
        if (strcmp((char *) encriptar, "1") == 0){

        for (i = 0; i < numbloques; i++)
        {
            //Dividimos en bloques los datos
            memcpy(block, &datos[i * tam_block], tam_block * sizeof(uint8_t));

            //Encriptamos el bloque
            process_message(init_vector, block_process, key_sets, 1);
            xor(block_process, block);
            memcpy(init_vector, block_process, tam_block * sizeof(uint8_t));

            //Copiamos el bloque en resultado
            memcpy(resultado + tam_block * i, block_process,
                   tam_block * sizeof(uint8_t));
        }
                       }else
                                      {
                                          for (i = 0; i < numbloques; i++)
                                          {
                                              memcpy(block, &datos[i * tam_block],
                                                     tam_block * sizeof(uint8_t));

                                              //Encriptamos el bloque
                                              process_message(init_vector, block_process, key_sets, 1);
                                              memcpy(init_vector, block, tam_block * sizeof(uint8_t));

                                              xor(block_process, block);

                                              //Copiamos el bloque en resultado
                                              memcpy(resultado + tam_block * i, block_process,
                                                     tam_block * sizeof(uint8_t));
                                          }
                                      }
    }

    //xTotalTime = (xTaskGetTickCount() - xTimeBefore) * (1 / configTICK_RATE_HZ);
    xTotalTime = xTaskGetTickCount() - xTimeBefore;

    sprintf(pcWriteBuffer, "Tiempo: %d Tick * (1/%d) \r\n MODO:%s-OK", xTotalTime,
    configTICK_RATE_HZ, modo);
    //strncat(pcWriteBuffer, "\r\n-OK",strlen("\r\n-OK"));
    //Copiamos el resultado en datos
    memcpy(datos, resultado, numbloques * tam_block * sizeof(uint8_t));

    //Liberamos las variables
    vPortFree(block);
    vPortFree(block_aux);
    vPortFree(block_process);
    vPortFree(resultado);
    vPortFree(init_vector);
    vPortFree(key_sets);

    //Devolvemos pdFALSE puesto que no vamos a realizar mas operaciones de salida
    return pdFALSE;

}

/*
 * Encripta o desencripta usando el Algoritmo 3DES
 */
static BaseType_t prvTDESCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                 const char *pcCommandString)
{
    //Obtenemos el parametro para saber si encriptamos o desencriptamos
    unsigned char *encriptar;
    BaseType_t encriptarStringLength;
    encriptar = (unsigned char *) FreeRTOS_CLIGetParameter(
            pcCommandString, 1, &encriptarStringLength);
    encriptar[encriptarStringLength] = 0x00;

    TickType_t xTimeBefore, xTotalTime;
    xTimeBefore = xTaskGetTickCount();

    //Obtenemos el numero de bloques necesarios para encriptar o desencriptar
    int numbloques = tam_datos / tam_block + ((tam_datos % tam_block) ? 1 : 0);

    //Reservamos el tamao de memoria necesario para cada variable que vamos a utilizar
    uint8_t *block = (uint8_t*) pvPortMalloc(tam_block * sizeof(uint8_t));
    uint8_t *block_process = (uint8_t*) pvPortMalloc(
            tam_block * sizeof(uint8_t));
    uint8_t *block_aux = (uint8_t*) pvPortMalloc(tam_block * sizeof(uint8_t));
    uint8_t* resultado = (uint8_t*) pvPortMalloc(
            numbloques * tam_block * sizeof(uint8_t));

    //Hacemos una copia del vector
    uint8_t* init_vector = (unsigned char*) pvPortMalloc(
            tam_clave * sizeof(uint8_t));
    memcpy(init_vector, vector, tam_clave * sizeof(uint8_t));

    //inicilizamos la clave que vamos a utilizar
    int i = 0;

    //Si encriptamos

    if (strcmp((char *) modo, "ECB") == 0)
    { //MODO ECB
        if (strcmp((char *) encriptar, "1") == 0)
        {

            //Con cada Bloque
            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en los datos en bloques
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));

                //Encriptamos el bloque
                tdes(block, block_process, clave, 24, 1, 0);

                //Guardamos el bloque en la variable resultado
                //memcpy(resultado + tam_block * i, block_process,tam_block * sizeof(uint8_t));
                memcpy(&resultado[tam_block * i], block_process,
                       tam_block * sizeof(uint8_t));
            }
        }
        else
        {            // si desencriptamos

            //Con cada Bloque
            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en los datos en bloques
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));

                //Desencriptamos el bloque
                tdes(block, block_process, clave, 24, 0, 0);
                //Guardamos el bloque en la variable resultado
                //memcpy(resultado + tam_block * i, block_process,       tam_block * sizeof(uint8_t));
                memcpy(&resultado[tam_block * i], block_process,
                       tam_block * sizeof(uint8_t));
            }
        }
    }
    else if (strcmp((char *) modo, "CBC") == 0)
    { //MODO CBC
        if (strcmp((char *) encriptar, "1") == 0)
        {   //Encriptamos

            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                xor(block, init_vector);

                //Encriptamos el bloque
                tdes(block, block_process, clave, 24, 1, 0);
                memcpy(init_vector, block_process, tam_clave * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));
            }
        }
        else
        {   //Desencriptamos

            for (i = 0; i < numbloques; i++)
                        {
                            //Dividimos en bloques los datos
                            memcpy(block, &datos[i * tam_block],
                                   tam_block * sizeof(uint8_t));

                            //Desencriptamos el bloque
                            tdes(block, block_process, clave, 24, 0, 0);
                            xor(block_process, init_vector);
                            memcpy(init_vector, block, tam_clave * sizeof(uint8_t));

                            //Copiamos el bloque en resultado
                            memcpy(resultado + tam_block * i, block_process,
                                   tam_block * sizeof(uint8_t));





                        }
        }

    }
    else if (strcmp((char *) modo, "PCB") == 0)
    { //MODO PCBC
        if (strcmp((char *) encriptar, "1") == 0)
        {   //Encriptamos

            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                memcpy(block_aux, block, tam_block * sizeof(uint8_t));
                xor(block, init_vector);

                //Encriptamos el bloque
                tdes(block, block_process, clave, 24, 1, 0);
                xor(block_aux, block_process);
                memcpy(init_vector, block_aux, tam_block * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));

            }
        }
        else
        {   //Desencriptamos

            for (i = 0; i < numbloques; i++)
            {   //Encriptamos

                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                memcpy(block_aux, block, tam_block * sizeof(uint8_t));

                //Desencriptamos el bloque
                tdes(block, block_process, clave, 24, 0, 0);
                xor(block_process, init_vector);
                xor(block_aux, block_process);
                memcpy(init_vector, block_aux, tam_block * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));

            }
        }
    }
    else if (strcmp((char *) modo, "OFB") == 0)
    {   //MODO OFB
        //En este modo da igual encriptar o desencriptar
        for (i = 0; i < numbloques; i++)
        {
            //Dividimos en bloques los datos
            memcpy(block, &datos[i * tam_block], tam_block * sizeof(uint8_t));

            //Encriptamos el bloque
            tdes(init_vector, block_process, clave, 24, 1, 0);
            memcpy(init_vector, block_process, tam_block * sizeof(uint8_t));
            xor(block_process, block);

            //Copiamos el bloque en resultado
            memcpy(resultado + tam_block * i, block_process,
                   tam_block * sizeof(uint8_t));
        }

    }
    else if (strcmp((char *) modo, "CFB") == 0)
    {   //MODO CFB
        if (strcmp((char *) encriptar, "1") == 0)
                {
        for (i = 0; i < numbloques; i++)
        {
            //Dividimos en bloques los datos
            memcpy(block, &datos[i * tam_block], tam_block * sizeof(uint8_t));

            //Encriptamos el bloque
            tdes(init_vector, block_process, clave, 24, 1, 0);
            xor(block_process, block);
            memcpy(init_vector, block_process, tam_block * sizeof(uint8_t));

            //Copiamos el bloque en resultado
            memcpy(resultado + tam_block * i, block_process,
                   tam_block * sizeof(uint8_t));
        }
                }else
                {
                    for (i = 0; i < numbloques; i++)
                    {
                        memcpy(block, &datos[i * tam_block],
                               tam_block * sizeof(uint8_t));

                        //Encriptamos el bloque
                        tdes(init_vector, block_process, clave, 24, 1, 0);
                        memcpy(init_vector, block, tam_block * sizeof(uint8_t));

                        xor(block_process, block);

                        //Copiamos el bloque en resultado
                        memcpy(resultado + tam_block * i, block_process,
                               tam_block * sizeof(uint8_t));
                    }
                }

    }

    //Devolvemos ok
    //sprintf(pcWriteBuffer, "\r\n-OK");
    xTotalTime = xTaskGetTickCount() - xTimeBefore;
    sprintf(pcWriteBuffer, "Tiempo: %d Tick * (1/%d) \r\n-OK", xTotalTime,
    configTICK_RATE_HZ);
    //Copiamos el resultado en Datos.
    memcpy(datos, resultado, numbloques * tam_block * sizeof(uint8_t));

    //Liberamos las variables utilizadas
    vPortFree(block);
    vPortFree(block_process);
    vPortFree(block_aux);
    vPortFree(resultado);
    vPortFree(init_vector);

    return pdFALSE;

}

/*
 * Encripta o desencripta usando el Algoritmo AES Software
 */

static BaseType_t prvAESCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                const char *pcCommandString)
{
    //Obtenemos el parametro para saber si encriptamos o desencriptamos
    unsigned char *encriptar;
    BaseType_t encriptarStringLength;
    encriptar = (unsigned char *) FreeRTOS_CLIGetParameter(
            pcCommandString, 1, &encriptarStringLength);
    encriptar[encriptarStringLength] = 0x00;

    TickType_t xTimeBefore, xTotalTime;
    xTimeBefore = xTaskGetTickCount();

    //Obtenemos el numero de bloques necesarios para encriptar o desencriptar
    int numbloques = tam_datos / tam_block + ((tam_datos % tam_block) ? 1 : 0);

    //Reservamos el tamao de memoria necesario para cada variable que vamos a utilizar
    uint8_t *block = (uint8_t*) pvPortMalloc(tam_block * sizeof(uint8_t));
    uint8_t *block_process = (uint8_t*) pvPortMalloc(
            tam_block * sizeof(uint8_t));
    uint8_t *block_aux = (uint8_t*) pvPortMalloc(tam_block * sizeof(uint8_t));
    uint8_t* resultado = (uint8_t*) pvPortMalloc(
            numbloques * tam_block * sizeof(uint8_t));

    //Hacemos una copia del vector
    uint8_t* init_vector = (unsigned char*) pvPortMalloc(
            tam_clave * sizeof(uint8_t));
    memcpy(init_vector, vector, tam_clave * sizeof(uint8_t));

    //inicilizamos la clave que vamos a utilizar
    WORD key_schedule[60];
    int tam_aes = 0;
    if (tam_clave == 16)
    {
        tam_aes = 128;
    }
    else if (tam_clave == 24)
    {
        tam_aes = 192;
    }
    else if (tam_clave == 32)
    {
        tam_aes = 256;
    }

    aes_key_setup(clave, key_schedule, tam_aes);
    int i = 0;

    //sprintf(pcWriteBuffer, "Datos - %s - %d\r\n", datos, tam_datos);
//MAP_WDT_A_holdTimer();

    //Si encriptamos

    if (strcmp((char *) modo, "ECB") == 0)
    { //MODO ECB
        if (strcmp((char *) encriptar, "1") == 0)
        {

            //Con cada Bloque
            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en los datos en bloques
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));

                //Encriptamos el bloque
                aes_encrypt(block, block_process, key_schedule, tam_aes);

                //Guardamos el bloque en la variable resultado
                //memcpy(resultado + tam_block * i, block_process,tam_block * sizeof(uint8_t));
                memcpy(&resultado[tam_block * i], block_process,
                       tam_block * sizeof(uint8_t));
            }
        }
        else
        {            // si desencriptamos

            //Con cada Bloque
            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en los datos en bloques
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));

                //Desencriptamos el bloque
                aes_decrypt(block, block_process, key_schedule, tam_aes);
                //Guardamos el bloque en la variable resultado
                //memcpy(resultado + tam_block * i, block_process,       tam_block * sizeof(uint8_t));
                memcpy(&resultado[tam_block * i], block_process,
                       tam_block * sizeof(uint8_t));
            }
        }
    }
    else if (strcmp((char *) modo, "CBC") == 0)
    { //MODO CBC
        if (strcmp((char *) encriptar, "1") == 0)
        {   //Encriptamos

            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                xor(block, init_vector);

                //Encriptamos el bloque
                aes_encrypt(block, block_process, key_schedule, tam_aes);
                memcpy(init_vector, block_process, tam_clave * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));
            }
        }
        else
        {   //Desencriptamos

            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));

                //Desencriptamos el bloque
                aes_decrypt(block, block_process, key_schedule, tam_aes);
                xor(block_process, init_vector);
                memcpy(init_vector, block, tam_clave * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));
            }
        }

    }
    else if (strcmp((char *) modo, "PCB") == 0)
    { //MODO PCBC
        if (strcmp((char *) encriptar, "1") == 0)
        {   //Encriptamos

            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                memcpy(block_aux, block, tam_block * sizeof(uint8_t));
                xor(block, init_vector);

                //Encriptamos el bloque
                aes_encrypt(block, block_process, key_schedule, tam_aes);
                xor(block_aux, block_process);
                memcpy(init_vector, block_aux, tam_block * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));

            }
        }
        else
        {   //Desencriptamos

            for (i = 0; i < numbloques; i++)
            {   //Encriptamos

                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                memcpy(block_aux, block, tam_block * sizeof(uint8_t));

                //Desencriptamos el bloque
                aes_decrypt(block, block_process, key_schedule, tam_aes);
                xor(block_process, init_vector);
                xor(block_aux, block_process);
                memcpy(init_vector, block_aux, tam_block * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));

            }
        }
    }
    else if (strcmp((char *) modo, "OFB") == 0)
    {   //MODO OFB
        //En este modo da igual encriptar o desencriptar
        for (i = 0; i < numbloques; i++)
        {
            //Dividimos en bloques los datos
            memcpy(block, &datos[i * tam_block], tam_block * sizeof(uint8_t));

            //Encriptamos el bloque
            aes_encrypt(init_vector, block_process, key_schedule, tam_aes);
            memcpy(init_vector, block_process, tam_block * sizeof(uint8_t));
            xor(block_process, block);

            //Copiamos el bloque en resultado
            memcpy(resultado + tam_block * i, block_process,
                   tam_block * sizeof(uint8_t));
        }

    }
    else if (strcmp((char *) modo, "CFB") == 0)
    {   //MODO CFB
        if (strcmp((char *) encriptar, "1") == 0)
        {
        for (i = 0; i < numbloques; i++)
        {
            //Dividimos en bloques los datos
            memcpy(block, &datos[i * tam_block], tam_block * sizeof(uint8_t));

            //Encriptamos el bloque
            aes_encrypt(init_vector, block_process, key_schedule, tam_aes);
            xor(block_process, block);
            memcpy(init_vector, block_process, tam_block * sizeof(uint8_t));

            //Copiamos el bloque en resultado
            memcpy(resultado + tam_block * i, block_process,
                   tam_block * sizeof(uint8_t));
        }
        }else
        {
            for (i = 0; i < numbloques; i++)
            {
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));

                //Encriptamos el bloque
                aes_encrypt(init_vector, block_process, key_schedule, tam_aes);
                memcpy(init_vector, block, tam_block * sizeof(uint8_t));

                xor(block_process, block);

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));
            }
        }

    }

    //Devolvemos ok
    //sprintf(pcWriteBuffer, "\r\n-OK");
    xTotalTime = xTaskGetTickCount() - xTimeBefore;
    sprintf(pcWriteBuffer, "Tiempo: %d Tick * (1/%d) \r\n-OK", xTotalTime,
    configTICK_RATE_HZ);
    //Copiamos el resultado en Datos.
    memcpy(datos, resultado, numbloques * tam_block * sizeof(uint8_t));

    //Liberamos las variables utilizadas
    vPortFree(block);
    vPortFree(block_process);
    vPortFree(block_aux);
    vPortFree(resultado);
    vPortFree(init_vector);

    return pdFALSE;

}

/*
 * Encripta o desencripta usando el Algoritmo AES Hardware
 */

static BaseType_t prvAES_HARDCommand(char *pcWriteBuffer,
                                     size_t xWriteBufferLen,
                                     const char *pcCommandString)
{
    //Obtenemos el parametro que nos indica si encriptamos o desencriptamos
    unsigned char *encriptar;
    BaseType_t encriptarStringLength;
    encriptar = (unsigned char *) FreeRTOS_CLIGetParameter(
            pcCommandString, 1, &encriptarStringLength);
    encriptar[encriptarStringLength] = 0x00;

    TickType_t xTimeBefore, xTotalTime;
    xTimeBefore = xTaskGetTickCount();

    //Obtenemos el numero de bloques que tenemos que encriptar
    int numbloques = tam_datos / tam_block + ((tam_datos % tam_block) ? 1 : 0);

    //Reservamos la memoria para las variables que vamos a utilizar
    uint8_t *block = (uint8_t*) pvPortMalloc(tam_block * sizeof(uint8_t));
    uint8_t *block_process = (uint8_t*) pvPortMalloc(
            tam_block * sizeof(uint8_t));
    uint8_t *block_aux = (uint8_t*) pvPortMalloc(tam_block * sizeof(uint8_t));
    uint8_t* resultado = (uint8_t*) pvPortMalloc(
            numbloques * tam_block * sizeof(uint8_t));

    int i = 0;

    int tam_aes = 0;
    if (tam_clave == 16)
    {
        tam_aes = AES256_KEYLENGTH_128BIT;
    }
    else if (tam_clave == 24)
    {
        tam_aes = AES256_KEYLENGTH_192BIT;
    }
    else if (tam_clave == 32)
    {
        tam_aes = AES256_KEYLENGTH_256BIT;
    }

    uint8_t* init_vector = (unsigned char*) pvPortMalloc(
            tam_clave * sizeof(uint8_t));
    memcpy(init_vector, vector, tam_clave * sizeof(uint8_t));

    //sprintf(pcWriteBuffer, "Datos - %s - %d\r\n", datos, tam_datos);

    //Paramos el Watchdog
    MAP_WDT_A_holdTimer();

    if (strcmp((char *) modo, "ECB") == 0)
    {   //Modo ECB
        //si es uno encriptamos
        if (strcmp((char *) encriptar, "1") == 0)
        {
            //Preparamos la clave
            MAP_AES256_setCipherKey(AES256_BASE, clave, tam_aes);

            for (i = 0; i < numbloques; i++)
            {
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                MAP_AES256_encryptData(AES256_BASE, block, block_process);
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));
            }
        }
        else
        {            //si es cero desencriptamos

            //Preparamos la clave
            MAP_AES256_setDecipherKey(AES256_BASE, clave, tam_aes);

            for (i = 0; i < numbloques; i++)
            {
                //Dividimos los datos en Bloques de tamao tam_block
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                //Encriptamso los datos
                MAP_AES256_decryptData(AES256_BASE, block, block_process);
                //Guardamos el resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));
            }
        }
    }
    else if (strcmp((char *) modo, "CBC") == 0)
    {            //MODO CBC
        if (strcmp((char *) encriptar, "1") == 0)
        {   //Encriptamos
            MAP_AES256_setCipherKey(AES256_BASE, clave, tam_aes);

            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                xor(block, init_vector);

                //Encriptamos el bloque
                MAP_AES256_encryptData(AES256_BASE, block, block_process);
                memcpy(init_vector, block_process, tam_clave * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));
            }
        }
        else
        {   //Desencriptamos
            MAP_AES256_setDecipherKey(AES256_BASE, clave, tam_aes);
            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));

                //Desencriptamos el bloque
                MAP_AES256_decryptData(AES256_BASE, block, block_process);
                xor(block_process, init_vector);
                memcpy(init_vector, block, tam_clave * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));
            }
        }

    }
    else if (strcmp((char *) modo, "PCB") == 0)
    {   //MODO PCBC
        if (strcmp((char *) encriptar, "1") == 0)
        {   //Encriptamos
            MAP_AES256_setCipherKey(AES256_BASE, clave, tam_aes);

            for (i = 0; i < numbloques; i++)
            {
                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                memcpy(block_aux, block, tam_block * sizeof(uint8_t));
                xor(block, init_vector);

                //Encriptamos el bloque
                MAP_AES256_encryptData(AES256_BASE, block, block_process);
                xor(block_aux, block_process);
                memcpy(init_vector, block_aux, tam_block * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));

            }
        }
        else
        {   //Desencriptamos
            MAP_AES256_setDecipherKey(AES256_BASE, clave, tam_aes);
            for (i = 0; i < numbloques; i++)
            {   //Encriptamos

                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));
                memcpy(block_aux, block, tam_block * sizeof(uint8_t));

                //Desencriptamos el bloque
                MAP_AES256_decryptData(AES256_BASE, block, block_process);
                xor(block_process, init_vector);
                xor(block_aux, block_process);
                memcpy(init_vector, block_aux, tam_block * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));

            }
        }
    }
    else if (strcmp((char *) modo, "OFB") == 0)
    {   //MODO OFB

        MAP_AES256_setCipherKey(AES256_BASE, clave, tam_aes);
        //En este modo da igual encriptar o desencriptar
        for (i = 0; i < numbloques; i++)
        {

            //Dividimos en bloques los datos
            memcpy(block, &datos[i * tam_block], tam_block * sizeof(uint8_t));

            //Encriptamos el bloque
            MAP_AES256_encryptData(AES256_BASE, init_vector, block_process);
            memcpy(init_vector, block_process, tam_block * sizeof(uint8_t));
            xor(block_process, block);

            //Copiamos el bloque en resultado
            memcpy(resultado + tam_block * i, block_process,
                   tam_block * sizeof(uint8_t));
        }

    }
    else if (strcmp((char *) modo, "CFB") == 0)
    {   //MODO CFB

        MAP_AES256_setCipherKey(AES256_BASE, clave, tam_aes);

        if (strcmp((char *) encriptar, "1") == 0)
        {
            for (i = 0; i < numbloques; i++)
            {

                //Dividimos en bloques los datos
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));

                //Encriptamos el bloque
                MAP_AES256_encryptData(AES256_BASE, init_vector, block_process);
                xor(block_process, block);
                memcpy(init_vector, block_process, tam_block * sizeof(uint8_t));

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));
            }
        }
        else
        {
            for (i = 0; i < numbloques; i++)
            {
                memcpy(block, &datos[i * tam_block],
                       tam_block * sizeof(uint8_t));

                //Encriptamos el bloque
                MAP_AES256_encryptData(AES256_BASE, init_vector, block_process);
                memcpy(init_vector, block, tam_block * sizeof(uint8_t));

                xor(block_process, block);

                //Copiamos el bloque en resultado
                memcpy(resultado + tam_block * i, block_process,
                       tam_block * sizeof(uint8_t));
            }
        }

    }

    //sacamos por pantalla ok
    //sprintf(pcWriteBuffer, "\r\n-OK");

    xTotalTime = xTaskGetTickCount() - xTimeBefore;
    sprintf(pcWriteBuffer, "Tiempo: %d Tick * (1/%d) \r\n-OK", xTotalTime,
    configTICK_RATE_HZ);

    //Guardamos resultado en datos
    memcpy(datos, resultado, numbloques * tam_block * sizeof(uint8_t));

    //Liberamos las variables utilizadas
    vPortFree(block);
    vPortFree(block_process);
    vPortFree(resultado);
    vPortFree(init_vector);
    vPortFree(block_aux);
    return pdFALSE;

}

/*
 * Generamos una llave aleatoria
 */

static BaseType_t prvGenKeyCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                   const char *pcCommandString)
{
    unsigned char *tam_key;
    BaseType_t tam_keyStringLength;
    tam_key = (unsigned char *) FreeRTOS_CLIGetParameter(pcCommandString, 1,
                                                         &tam_keyStringLength);
    tam_key[tam_keyStringLength] = 0x00;

    int t = atoi((char *) tam_key);
    if (t > 0)
    {

        int i;
        int c;
        //char a[t+1];
        uint8_t *key = (uint8_t*) pvPortMalloc(t * sizeof(uint8_t));
        uint8_t *key_hex = (uint8_t*) pvPortMalloc(2 * t * sizeof(uint8_t));
        srand(time(NULL));
        for (i = 0; i < t; i++)
        {
            c = rand() % 255;
            key[i] = c;
        }
        toStr(key, key_hex, t);

        sprintf(pcWriteBuffer, "Clave Generada: %s-OK\r\n", key_hex);
        vPortFree(key);
        vPortFree(key_hex);
    }
    else
    {
        sprintf(pcWriteBuffer, "Error al generar la clave-OK\r\n");
    }
    return pdFALSE;
}

/*
 * Hace xor sobre los parametros que le pasamos.
 */
void xor(uint8_t* str, uint8_t* str2)
{

    int i;

    for (i = 0; i < tam_block; i++)
    {
        uint8_t temp = str[i] ^ str2[i];
        str[i] = temp;
    }

}

/*
 * Comprueba si el parametro recibido coincide con el MD5 de los datos.
 */

static BaseType_t prvMD5Command(char *pcWriteBuffer, size_t xWriteBufferLen,
                                const char *pcCommandString)
{
    unsigned char *pcParameter1;
    BaseType_t xParameter1StringLength;
    pcParameter1 = (unsigned char *) FreeRTOS_CLIGetParameter(
            pcCommandString, 1, &xParameter1StringLength);

    pcParameter1[xParameter1StringLength] = 0x00;

    unsigned char* aux = (unsigned char*) pvPortMalloc(32 * sizeof(char));
    uint8_t* pcmd5 = (uint8_t*) pvPortMalloc(16 * sizeof(uint8_t));

    if (datos != NULL)
    {
        //Hace el md5 de los datos
        md5(datos, tam_datos_sin_pad, pcmd5);

        //convertimos los datos a Hexadecimal.
        int i = 0;
        for (i = 0; i < 16; i++)
        {
            sprintf((char *) aux + i * 2, "%02x", pcmd5[i]);
        }
        aux[32] = 0x00;

        //Comprobamos si coinciden.
        if ((strcmp((char *) pcParameter1, (char *) aux) == 0))
        {
            sprintf(pcWriteBuffer, "%s - %s -OK\n", aux, pcParameter1);
        }
        else
        {
            sprintf(pcWriteBuffer, "%s - %s %d-KO\n", aux, pcParameter1,
                    tam_datos_sin_pad);
        }

    }
    else
    {
        sprintf(pcWriteBuffer, "Los Datos estan vacos -KO\n");
    }
    //Devolvemos FALSE para
    vPortFree(aux);
    vPortFree(pcmd5);
    return pdFALSE;
}

/*
 * Transforma la cadena de Numeros a Caracteres en Hexadecimal
 */
void toStr(uint8_t* hex, unsigned char* str, int len)
{
    int i = 0;
//for (i = 0; i < strlen(hex); i++)
    for (i = 0; i < len; i++)
    {
        sprintf((char *) str + i * 2, "%02X", hex[i]);
    }

}

/*
 * Transforma Cadenas con caracteres en Hexadecimal en Cadenas uint8_t;
 */
void toUint8(uint8_t* hex, unsigned char* str, int len)
{
    int i = 0;
    for (i = 0; i < len / 2; i++)
    {
        sscanf((char*) hex + 2 * i, "%02x", &str[i]);
        //printf("bytearray %d: %02x\n", i, bytearray[i]);
    }

}

/*
 * Realiza la funcion Xor entre los parametros recibidos
 */

static BaseType_t prvXORCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                const char *pcCommandString)
{

    unsigned char *pcParameter1, *pcParameter2;
//*pcParameter5;

    BaseType_t xParameter1StringLength, xParameter2StringLength;

    pcParameter1 = (unsigned char *) FreeRTOS_CLIGetParameter(
            pcCommandString, 1, &xParameter1StringLength);
    pcParameter2 = (unsigned char *) FreeRTOS_CLIGetParameter(
            pcCommandString, 2, &xParameter2StringLength);

    pcParameter1[xParameter1StringLength] = 0x00;
    pcParameter2[xParameter2StringLength] = 0x00;
    unsigned char* bloque = (unsigned char*) pvPortMalloc(
            8 * sizeof(unsigned char));
    strncpy((char *) bloque, (char *) pcParameter1, 8);
    bloque[8] = 0x00;
    xor(bloque, pcParameter2);

    sprintf(pcWriteBuffer, "parametro1 %s - parametro2 %s - xor %s-OK\n",
            pcParameter1, pcParameter2, bloque);
    return pdFALSE;

}

/*
 * Devuelve el checksum de los datos
 */

void prvMD5(char *checksum)
{

    //unsigned char* aux = (unsigned char*) pvPortMalloc(32 * sizeof(char));
    uint8_t* pcmd5 = (uint8_t*) pvPortMalloc(16 * sizeof(uint8_t));

    if (datos != NULL)
    {
        md5(datos, tam_datos, pcmd5);
        //Convertimos a Hexadecimal
        int i = 0;
        for (i = 0; i < 16; i++)
        {
            sprintf(checksum + i * 2, "%02x", pcmd5[i]);
        }
        //aux[32] = 0x00;

    }

// Limpiamos aux
    //vPortFree(aux);
    vPortFree(pcmd5);

}

static BaseType_t prvRSACommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                const char *pcCommandString)
{
    unsigned char *encriptar;
    BaseType_t xencriptarLength;
    encriptar = (unsigned char *) FreeRTOS_CLIGetParameter(pcCommandString, 1,
                                                           &xencriptarLength);

    TickType_t xTimeBefore, xTotalTime;
    xTimeBefore = xTaskGetTickCount();

    //long* mens_long=(long*) pvPortMalloc( num_bloques * sizeof(long)); ;
    //long *mens_long = pvPortMalloc(sizeof(long)*num_bloques);

    if (strcmp((char *) encriptar, "1") == 0)
    {

        long *encriptado = rsa_encrypt(datos, tam_datos_sin_pad, pub);
        int i = 0;

        char temp[5] = "%0";
        char aux[1];
        sprintf(aux, "%d", tam_block);
        strcat(temp, aux);
        strcat(temp, "d");

        unsigned char* cadena = (unsigned char*) pvPortMalloc(
                tam_block * sizeof(unsigned char));
        for (i = 0; i < tam_datos_sin_pad; i++)
        {

            sprintf((char *) cadena, temp, encriptado[i]);
            memcpy(datos + tam_block * i, cadena, tam_block * sizeof(uint8_t));
        }

        xTotalTime = xTaskGetTickCount() - xTimeBefore;
        sprintf(pcWriteBuffer,
                "Mensaje encriptado\r\nTiempo: %d Tick * (1/%d) \r\n  tam datos %d-OK",
                xTotalTime, configTICK_RATE_HZ, tam_datos);
        //tam_datos = tam_block * tam_datos;
        vPortFree(encriptado);
        vPortFree(cadena);

    }

    else
    {   //int tam_block=10;
        int num_bloques = tam_datos / tam_block;
        unsigned char *block = (unsigned char*) pvPortMalloc(
                tam_block * sizeof(unsigned char));
        unsigned char *decrypted = (unsigned char*) pvPortMalloc(
                num_bloques * sizeof(unsigned char));

        int i = 0;
        for (i = 0; i < num_bloques; i++)
        {
            memcpy(block, &datos[i * tam_block], tam_block * sizeof(uint8_t));
            //mens_long[i]=atol(block);
            decrypted[i] = rsa_modExp(atol((char *) block), pri->exponent,
                                      pri->modulus);
        }

        //char *decrypted =rsa_decrypt(mens_long, num_bloques, pub);
        memset(datos, 0x00, tam_datos);
        memcpy(datos, decrypted, num_bloques * sizeof(uint8_t));
        tam_datos = num_bloques;

        xTotalTime = xTaskGetTickCount() - xTimeBefore;
        sprintf(pcWriteBuffer,
                "Mensaje desencriptado\r\nTiempo: %d Tick * (1/%d) \r\n-OK",
                xTotalTime, configTICK_RATE_HZ);
        vPortFree(block);
        vPortFree(decrypted);

    }
    return pdFALSE;
}

static BaseType_t prvSetPublicKeyCommand(char *pcWriteBuffer,
                                         size_t xWriteBufferLen,
                                         const char *pcCommandString)
{
    unsigned char *Key, *modulus;
//*pcParameter5;

    BaseType_t xKeyStringLength, xmodulusStringLength;

    Key = (unsigned char *) FreeRTOS_CLIGetParameter(pcCommandString, 1,
                                                     &xKeyStringLength);
    modulus = (unsigned char *) FreeRTOS_CLIGetParameter(pcCommandString, 2,
                                                         &xmodulusStringLength);

    Key[xKeyStringLength] = 0x00;
    modulus[xmodulusStringLength] = 0x00;

    pub->exponent = atol((char*) Key);
    pub->modulus = atol((char*) modulus);
    tam_block = numerodigitos(pub->modulus);

    sprintf(pcWriteBuffer, "Claves Publicas Correctas %d %d-OK", pub->exponent,
            pub->modulus);

    //Devolvemos FALSE para
    return pdFALSE;
}

static BaseType_t prvGetPublicKeyCommand(char *pcWriteBuffer,
                                         size_t xWriteBufferLen,
                                         const char *pcCommandString)
{

    //Generamos la clave plublica y privada
    rsa_gen_keys(pub, pri);
    //Obtenemos el tamao del bloque ya que el numero no debe ser mayor del modulo
    tam_block = numerodigitos(pub->modulus);

    //sprintf(pcWriteBuffer, "Clave Publica: %d\r\nModulo:%d\r\nClave Privada: %d\r\nModulo Privado:%d\r\n-OK",pub->exponent,pub->modulus, pri->exponent,pri->modulus);
    sprintf(pcWriteBuffer, "Clave Publica: %d\r\nModulo:%d\r\n-OK",
            pub->exponent, pub->modulus);

    //Devolvemos FALSE para
    return pdFALSE;
}

static BaseType_t prvTextRSACommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                    const char *pcCommandString)
{
    unsigned char *aux;
    BaseType_t len;

    aux = (unsigned char *) FreeRTOS_CLIGetParameter(pcCommandString, 1, &len);
    aux[len] = 0x00;

    //Si el tamao que queda de buffer es igual o mayor que el tamao del parametro lo aadimos.
    if (size + len <= tam_datos)
    {
        //Si datos esta inicializado aadimos el parametro recibido
        if (datos != NULL)
        {

            //Copiamos los aux al vector de datos
            memcpy(&datos[size], aux, len * sizeof(uint8_t));
            //memcpy(datos+size, bytearray, len * sizeof(uint8_t));

            // Sumamos el tamao aadido
            size = size + len;
            sprintf(pcWriteBuffer, "%d - %d -OK", tam_datos, size);

        }
        else
        {   //Devolmemos el error
            sprintf(pcWriteBuffer, "ERROR - Datos no iniciados -KO");
        }
    }
    else
    {
        sprintf(pcWriteBuffer,
                "ERROR - Los datos son mayores que el buffer -KO");
    }

    return pdFALSE;

}

static BaseType_t prvIniRSACommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                   const char *pcCommandString)
{
    unsigned char *prmTamDatos;
    BaseType_t xPTamDatosLength;
    prmTamDatos = (unsigned char *) FreeRTOS_CLIGetParameter(pcCommandString, 1,
                                                             &xPTamDatosLength);
    unsigned char *prmTipo;
        BaseType_t xPTipoLength;
        prmTipo = (unsigned char *) FreeRTOS_CLIGetParameter(pcCommandString, 2,
                                                                 &xPTipoLength);

    prmTamDatos[xPTamDatosLength] = 0x00;
    prmTipo[xPTipoLength] = 0x00;

    size = 0;
    if (datos != NULL)
    {
        //datos = NULL;
        vPortFree(datos);
    }
    if (strcmp((char *) prmTipo, "0") == 0){
        tam_datos = atoi((char *) prmTamDatos);
        tam_datos_sin_pad = atoi((char *) prmTamDatos);
    }else{
        tam_datos = atoi((char *) prmTamDatos)*tam_block;
        tam_datos_sin_pad = atoi((char *) prmTamDatos);
    }


    datos = (uint8_t*) pvPortMalloc(tam_datos * sizeof(uint8_t));
    memset(datos, 0x00, tam_datos);
    sprintf(pcWriteBuffer, "Buffer creado-OK\n");
    return pdFALSE;
}

int numerodigitos(long num)
{
    int i = 0;
    //Dividimos entre 10 hasta que sea cero
    while (num != 0)
    {
        num = num / 10;
        i++;
    }
    return i;
}

static BaseType_t prvLOGINCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                  const char *pcCommandString)
{
    unsigned char contrasea[] = "Contrasea";
    unsigned char *pass;
    BaseType_t xpassLength;
    pass = (unsigned char *) FreeRTOS_CLIGetParameter(pcCommandString, 1,
                                                      &xpassLength);

    pass[xpassLength] = 0x00;

    //Si ya estas logeado mostramos un  mensaje de aviso
    if (login == 1)
    {
        sprintf(pcWriteBuffer, "YA ESTAS LOGEADO-OK\n");
    }
    else
    {
        //Obtenemos el numero de bloques enviados

        int num_bloques = xpassLength / tam_block;

        //Reservamos espacio para las operaciones

        unsigned char *block = (unsigned char*) pvPortMalloc(
                tam_block * sizeof(unsigned char));
        unsigned char *decrypted = (unsigned char*) pvPortMalloc(
                num_bloques + 1 * sizeof(unsigned char));

        int i = 0;
        //Desciframos el mensaje
        for (i = 0; i < num_bloques; i++)
        {
            memcpy(block, &pass[i * tam_block], tam_block * sizeof(uint8_t));
            decrypted[i] = rsa_modExp(atol((char *) block), pri->exponent,
                                      pri->modulus);
        }
        decrypted[num_bloques + 1] = 0x00;

        //Si coinciden ambas contraseas cambiamos el color de los led
        if (strcmp((char *) contrasea, (char *) decrypted) == 0)
        {
            login = 1;
            configTOGGLE_LED();

            GPIO_toggleOutputOnPin( GPIO_PORT_P2, GPIO_PIN1);
            GPIO_toggleOutputOnPin( GPIO_PORT_P2, GPIO_PIN0);
            sprintf(pcWriteBuffer, "CONTRASEA CORRECTA-OK\n");
        }
        else
        {
            //Mostramos un mensaje de error
            sprintf(pcWriteBuffer, "CONTRASEA ERRONEA-KO\n");
        }
    }

    return pdFALSE;
}

static BaseType_t prvLOGOUTCommand(char *pcWriteBuffer, size_t xWriteBufferLen,
                                   const char *pcCommandString)
{
    //Si esta logeado cambiamos los leda rojo  y la variable login a 0
    if (login == 1)
    {
        login = 0;
        configTOGGLE_LED();

        GPIO_toggleOutputOnPin( GPIO_PORT_P2, GPIO_PIN1);
        GPIO_toggleOutputOnPin( GPIO_PORT_P2, GPIO_PIN0);
        sprintf(pcWriteBuffer, "LOGOUT CORRECTO-OK\n");
    }
    else
    {

        //Si no esta logeado mostramos un mensaje de error
        sprintf(pcWriteBuffer, "NO ESTABAS LOGEADO-KO\n");
    }

    return pdFALSE;
}

