/*********************************************************************
* Filename:   aes_soft.h
* Author:     Carlos
* Copyright:
*********************************************************************/

#ifndef AES_H
#define AES_H

/*************************** HEADER FILES ***************************/
#include <stddef.h>

/****************************** MACROS ******************************/
#define AES_BLOCK_SIZE 16               // Tamao del Bloque

/**************************** DATA TYPES ****************************/
typedef unsigned char BYTE;            // 8-bit byte
typedef unsigned int WORD;             // 32-bit word

/*********************** FUNCTION DECLARATIONS **********************/
///////////////////
// AES
///////////////////
// La configuracin de la clave debe realizarse antes de que se puedan utilizar las funciones
// de encriptacio y desencriptacion de AES.
void aes_key_setup(const BYTE key[],          // La clave, debe ser de 128, 192, o 256 bits
                   WORD w[],                  // Salida de la clave schedule para ser utilizada mas tarde
                   int keysize);              // Tamao de la clave 128, 192, or 256

void aes_encrypt(const BYTE in[],             // 16 bytes of Texto plano
                 BYTE out[],                  // 16 bytes of Texto cifrado
                 const WORD key[],            // Clave
                 int keysize);                // Tamao de la clave 128, 192, or 256

void aes_decrypt(const BYTE in[],             // 16 bytes of Texto cifrado
                 BYTE out[],                  // 16 bytes of Texto cifrado
                 const WORD key[],            // Clave
                 int keysize);                // Tamao de la clave 128, 192, or 256

///////////////////
// AES - CBC
///////////////////
int aes_encrypt_cbc(const BYTE in[],          // Texto Plano
                    size_t in_len,            // Debe ser un mltiplo de AES_BLOCK_SIZE
                    BYTE out[],               // Texto cifrado, de la misma longitud que el texto cifrado
                    const WORD key[],         // Clave
                    int keysize,              // Tamao de la clave 128, 192, or 256
                    const BYTE iv[]);         // Vector debe ser de tamao AES_BLOCK_SIZE


///////////////////
// AES - CTR
///////////////////
void increment_iv(BYTE iv[],                  // Must be a multiple of AES_BLOCK_SIZE
                  int counter_size);          // Bytes of the IV used for counting (low end)

void aes_encrypt_ctr(const BYTE in[],         // Texto plano
                     size_t in_len,           // Cualquier tamao
                     BYTE out[],              // Texto cifrado, de la misma longitud que el texto plano
                     const WORD key[],        // Clave
                     int keysize,             // Tamao de la clave 128, 192, or 256
                     const BYTE iv[]);        // Vector debe ser de tamao AES_BLOCK_SIZE

void aes_decrypt_ctr(const BYTE in[],         // Ciphertext
                     size_t in_len,           // Any byte length
                     BYTE out[],              // Texto cifrado, de la misma longitud que el texto plano
                     const WORD key[],        // clave
                     int keysize,             // Tamao de la clave 128, 192, or 256
                     const BYTE iv[]);        // Vector debe ser de tamao AES_BLOCK_SIZE


#endif   // AES_H
