/*
    FreeRTOS V9.0.0 - Copyright (C) 2016 Real Time Engineers Ltd.
    All rights reserved

    VISIT http://www.FreeRTOS.org TO ENSURE YOU ARE USING THE LATEST VERSION.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation >>>> AND MODIFIED BY <<<< the FreeRTOS exception.

    ***************************************************************************
    >>!   NOTE: The modification to the GPL is included to allow you to     !<<
    >>!   distribute a combined work that includes FreeRTOS without being   !<<
    >>!   obliged to provide the source code for proprietary components     !<<
    >>!   outside of the FreeRTOS kernel.                                   !<<
    ***************************************************************************

    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT ANY
    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
    FOR A PARTICULAR PURPOSE.  Full license text is available on the following
    link: http://www.freertos.org/a00114.html

    ***************************************************************************
     *                                                                       *
     *    FreeRTOS provides completely free yet professionally developed,    *
     *    robust, strictly quality controlled, supported, and cross          *
     *    platform software that is more than just the market leader, it     *
     *    is the industry's de facto standard.                               *
     *                                                                       *
     *    Help yourself get started quickly while simultaneously helping     *
     *    to support the FreeRTOS project by purchasing a FreeRTOS           *
     *    tutorial book, reference manual, or both:                          *
     *    http://www.FreeRTOS.org/Documentation                              *
     *                                                                       *
    ***************************************************************************

    http://www.FreeRTOS.org/FAQHelp.html - Having a problem?  Start by reading
    the FAQ page "My application does not run, what could be wrong?".  Have you
    defined configASSERT()?

    http://www.FreeRTOS.org/support - In return for receiving this top quality
    embedded software for free we request you assist our global community by
    participating in the support forum.

    http://www.FreeRTOS.org/training - Investing in training allows your team to
    be as productive as possible as early as possible.  Now you can receive
    FreeRTOS training directly from Richard Barry, CEO of Real Time Engineers
    Ltd, and the world's leading authority on the world's leading RTOS.

    http://www.FreeRTOS.org/plus - A selection of FreeRTOS ecosystem products,
    including FreeRTOS+Trace - an indispensable productivity tool, a DOS
    compatible FAT file system, and our tiny thread aware UDP/IP stack.

    http://www.FreeRTOS.org/labs - Where new FreeRTOS products go to incubate.
    Come and try FreeRTOS+TCP, our new open source TCP/IP stack for FreeRTOS.

    http://www.OpenRTOS.com - Real Time Engineers ltd. license FreeRTOS to High
    Integrity Systems ltd. to sell under the OpenRTOS brand.  Low cost OpenRTOS
    licenses offer ticketed support, indemnification and commercial middleware.

    http://www.SafeRTOS.com - High Integrity Systems also provide a safety
    engineered and independently SIL3 certified version for use in safety and
    mission critical applications that require provable dependability.

    1 tab == 4 spaces!
*/


/************************************************** ****************************
 * NOTA 1: Este proyecto proporciona dos aplicaciones de demostracin.
 * Un simple proyecto de estilo blinky, y una prueba ms completa y una aplicacin de demostracin.
 * La configuracin configCREATE_SIMPLE_TICKLESS_DEMO en FreeRTOSConfig.h se usa para seleccionar
 * entre los dos. Consulte las notas sobre el uso de configCREATE_SIMPLY_BLINKY_DEMO_ONLY en main.c.
 * Este archivo implementa la versin completa.
 *
 * NOTA 2: Este archivo solo contiene el cdigo fuente que es especfico para la demostracin completa.
 * Las funciones genricas, las funciones de enlace FreeRTOS y las funciones necesarias para configurar
 * el hardware se definen en main.c.
 *
 * Consulte http://www.FreeRTOS.org/TI_MSP432_Free_RTOS_Demo.html para obtener instrucciones.
 *
 ************************************************** ****************************
 *
 * main_full () crea todas las tareas de la aplicacin de demostracin y
 * los temporizadores de software, luego inicia el programador. La documentacin
 * web proporciona ms detalles de las tareas de la aplicacin de demostracin
 * estndar, que no proporcionan ninguna funcionalidad en particular, pero
 * proporcionan un buen ejemplo de cmo usar la API de FreeRTOS.
 *
 * Adems de las tareas de demostracin estndar, las siguientes tareas y
 * pruebas se definen y / o se crean dentro de este archivo:
 *
 * Consola de comandos de FreeRTOS + CLI. La consola de comandos tiene acceso a travs del UART a 19200 baudios.
 * Por razones de pruebas de robustez, el controlador UART se escribe deliberadamente para ser ineficiente y
 * no se debe usar como plantilla para un controlador de produccin. Escriba "ayuda" para ver una lista de comandos
 * registrados. La licencia de FreeRTOS + CLI es diferente a la licencia de FreeRTOS,
 * consulte http://www.FreeRTOS.org/cli para obtener detalles sobre la licencia y el uso.
 *
 * Tareas de "Reg Test"(prueba de registro): completan los registros de ncleo y punto flotante con valores conocidos,
 * luego verifican que cada registro mantenga su valor esperado durante el tiempo de vida de la tarea. Cada tarea usa
 * un conjunto diferente de valores. Las tareas de prueba de registro se ejecutan con una prioridad muy baja,
 * por lo que se apropia con mucha frecuencia. Un registro que contiene un valor
 * inesperado es indicativo de un error en el mecanismo de cambio de contexto.
 *
 * Tarea "Comprobar": el perodo de la tarea de verificacin se establece inicialmente en tres segundos.
 * La tarea comprueba que todas las tareas de demostracin estndar y las tareas de comprobacin de registros
 * no solo se estn ejecutando, sino que se estn ejecutando sin informar ningn error. Si la tarea de
 * verificacin descubre que una tarea se ha estancado o ha informado de un error, cambia su propio perodo de
 * ejecucin de los tres segundos iniciales a solo 200 ms. La tarea de verificacin tambin alterna un LED cada
 * vez que se llama. Esto proporciona una indicacin visual del estado del sistema: si el LED alterna cada tres
 * segundos, entonces no se han descubierto problemas. Si el LED alterna cada 200 ms, entonces se ha descubierto
 * un problema con al menos una tarea.
 */

/* Standard includes. */
#include <stdio.h>

/* Kernel includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "timers.h"
#include "semphr.h"

/* Standard demo application includes. */
#include "flop.h"
#include "semtest.h"
#include "countsem.h"
#include "GenQTest.h"
#include "recmutex.h"
#include "partest.h"
#include "serial.h"
#include "TimerDemo.h"
#include "IntQueue.h"
#include "EventGroupsDemo.h"
#include "TaskNotify.h"
#include "IntSemTest.h"

/* Prioridades para las tareas de la aplicacin de demostracin. */
#define mainSEM_TEST_PRIORITY				( tskIDLE_PRIORITY + 1UL )
#define mainBLOCK_Q_PRIORITY				( tskIDLE_PRIORITY + 2UL )
#define mainCREATOR_TASK_PRIORITY			( tskIDLE_PRIORITY + 3UL )
#define mainFLOP_TASK_PRIORITY				( tskIDLE_PRIORITY )
#define mainUART_COMMAND_CONSOLE_STACK_SIZE	( configMINIMAL_STACK_SIZE * 2UL )
#define mainCOM_TEST_TASK_PRIORITY			( tskIDLE_PRIORITY + 2 )
#define mainCHECK_TASK_PRIORITY				( configMAX_PRIORITIES - 1 )
#define mainQUEUE_OVERWRITE_PRIORITY		( tskIDLE_PRIORITY )

/* La prioridad utilizada por la tarea de la consola de comandos de UART. */
#define mainUART_COMMAND_CONSOLE_TASK_PRIORITY	( configMAX_PRIORITIES - 2 )

/* Un tiempo de bloqueo de cero simplemente significa "no bloquear". */
#define mainDONT_BLOCK						( 0UL )

/* El perodo de la tarea de verificacin, en ms, siempre que no se hayan informado
 * errores en ninguna de las tareas de demostracin estndar. ms se convierten al
 * equivalente en ticks usando la constante de macro pdMS_TO_TICKS (). */
#define mainNO_ERROR_CHECK_TASK_PERIOD		( pdMS_TO_TICKS( 3000UL ) )

/* El perodo de la tarea de verificacin, en ms, si se ha informado un error en una
 * de las tareas de demostracin estndar. ms se convierten a su equivalente en ticks
 * usando la macro pdMS_TO_TICKS (). */
#define mainERROR_CHECK_TASK_PERIOD 		( pdMS_TO_TICKS( 200UL ) )

/* Los parmetros que se pasan a las tareas de comprobacin de registro
 * con el nico fin de garantizar parmetros se pasan a las tareas correctamente. */
#define mainREG_TEST_TASK_1_PARAMETER		( ( void * ) 0x12345678 )
#define mainREG_TEST_TASK_2_PARAMETER		( ( void * ) 0x87654321 )

/* El perodo base utilizado por las tareas de prueba del temporizador. */
#define mainTIMER_TEST_PERIOD				( 5000 )

/* Dimensiones de la cola en la que se colocan los caracteres recibidos de UART. */
#define mainRX_QUEUE_LENGTH					10

/*-----------------------------------------------------------*/


/*
 * La tarea de verificacin, como se describe en la parte superior de este archivo.
 */
//static void prvCheckTask( void *pvParameters );

/*
 * Registre las tareas de verificacin y las tareas utilizadas para escribir y verificar
 * el contenido de los registros FPU, tal como se describe en la parte superior de este archivo.
 * La naturaleza de estos archivos requiere que estn escritos en un archivo de ensamblaje,
 * pero los puntos de entrada se mantienen en el archivo C para la conveniencia de verificar
 * el parmetro de la tarea.
 */
/*static void prvRegTestTaskEntry1( void *pvParameters );
extern void vRegTest1Implementation( void );
static void prvRegTestTaskEntry2( void *pvParameters );
extern void vRegTest2Implementation( void );*/

/*
 * Registre los comandos que se pueden usar con FreeRTOS + CLI.
 * Los comandos se definen en CLI-Commands.c y File-Related-CLI-Command.c respectivamente.
 */
//extern void vRegisterSampleCLICommands( void );
extern void vRegisterCLICommands( void );

/*
 * La tarea que gestiona la entrada y salida de FreeRTOS + CLI.
 */
extern void vUARTCommandConsoleStart( uint16_t usStackSize, UBaseType_t uxPriority );

/*
 * Cuando se construye la demostracin completa, el gancho de inactividad se usa para crear
 * algunos temporizadores que no se pueden crear en main () porque las tareas de demostracin
 * del temporizador necesitan toda la cola de comandos.
 */
void vFullDemoIdleHook( void );

/*
 * La demo completa configura los relojes para la frecuencia mxima, mientras que
 * la demo blinky usa un reloj ms lento ya que tambin usa caractersticas de baja potencia.
 */
static void prvConfigureClocks( void );

/*-----------------------------------------------------------*/

/* Las siguientes dos variables se usan para comunicar el estado de las tareas de verificacin
 * de registro a la tarea de verificacin. Si las variables siguen incrementndose,
 * las tareas de comprobacin de registro no han descubierto ningn error.
 * Si una variable deja de incrementarse, se ha encontrado un error. */
volatile unsigned long ulRegTest1LoopCounter = 0UL, ulRegTest2LoopCounter = 0UL;

/*-----------------------------------------------------------*/

void main_full( void )
{
	/* Esta demostracin establece el reloj al mximo. La demo blinky usa
	 * un reloj ms lento ya que usa funciones de baja potencia.  */
	prvConfigureClocks();

	/* Init the serial port for use by the CLI.  The baud rate parameter is not
	used so set to 0 to make this obvious. */
	xSerialPortInitMinimal( 0, mainRX_QUEUE_LENGTH );

	/* Comience todas las otras tareas estndar de demostracin / prueba. No tienen ninguna
	 * funcionalidad en particular, pero s demuestran cmo usar la API de FreeRTOS
	 * y probar el puerto del kernel. */
	vStartInterruptQueueTasks();

	vStartCountingSemaphoreTasks();
	vStartGenericQueueTasks( tskIDLE_PRIORITY );
	vStartRecursiveMutexTasks();
	vStartSemaphoreTasks( mainSEM_TEST_PRIORITY );
	vStartTimerDemoTask( mainTIMER_TEST_PERIOD );
	vStartEventGroupTasks();
	vStartTaskNotifyTask();
	vStartInterruptSemaphoreTasks();

	/* Nota: el conjunto de tareas de demostracin estndar contiene dos versiones de vStartMathTasks.c.
	 * Uno se define en flop.c y usa nmeros y variables de coma flotante de precisin doble.
	 * El otro se define en sp_flop.c, y usa nmeros y variables de coma flotante de precisin simple. sp_flop. c
	 * debe incluirse en este proyecto. */
	vStartMathTasks( mainFLOP_TASK_PRIORITY );

	/* Inicie las tareas que implementan la consola de comandos en el UART, como se describe arriba. */
	vUARTCommandConsoleStart( mainUART_COMMAND_CONSOLE_STACK_SIZE, mainUART_COMMAND_CONSOLE_TASK_PRIORITY );

	/* Registre los comandos estndares de la CLI. */
	//vRegisterSampleCLICommands();
	vRegisterCLICommands();

	/* Cree las tareas de verificacin de registro, tal como se describe en la parte superior de este archivo */
	//xTaskCreate( prvRegTestTaskEntry1, "Reg1", configMINIMAL_STACK_SIZE, mainREG_TEST_TASK_1_PARAMETER, tskIDLE_PRIORITY, NULL );
	//xTaskCreate( prvRegTestTaskEntry2, "Reg2", configMINIMAL_STACK_SIZE, mainREG_TEST_TASK_2_PARAMETER, tskIDLE_PRIORITY, NULL );

	/* Cree la tarea que realiza la funcionalidad 'verificar', como se describe en la parte superior de este archivo. */
	//xTaskCreate( prvCheckTask, "Check", configMINIMAL_STACK_SIZE, NULL, mainCHECK_TASK_PRIORITY, NULL );

	/* Inicie el programador. */
	vTaskStartScheduler();

	/* Si todo est bien, el programador ahora se ejecutar, y nunca se alcanzar la siguiente lnea.
	 * Si se ejecuta la lnea siguiente, entonces no haba suficiente memoria de montn de FreeRTOS
	 * disponible para crear las tareas de inactividad y / o de temporizador. Consulte la seccin
	 * de administracin de memoria en el sitio web de FreeRTOS para obtener ms detalles sobre el montn FreeRTOS
	 * http://www.freertos.org/a00111.html. */
	for( ;; );
}


static void prvConfigureClocks( void )
{
	/* Establezca el estado de espera de Flash para una frecuencia de reloj alta. Consulte la hoja de datos
	 * para ms detalles. */
	FlashCtl_setWaitState( FLASH_BANK0, 2 );
	FlashCtl_setWaitState( FLASH_BANK1, 2 );

	/* La demo completa configura los relojes para la frecuencia mxima, mientras que la demo blinky usa un
	 * reloj ms lento ya que tambin usa caractersticas de baja potencia. La frecuencia mxima tambin
	 * necesita ms voltaje.
	 *
	 * Desde el datashee: Para los modos AM_LDO_VCORE1 y AM_DCDC_VCORE1, la frecuencia mxima de operacin de la
	 * CPU es de 48 MHz y la frecuencia mxima del reloj de entrada para los perifricos es de 24 MHz. */
	PCM_setCoreVoltageLevel( PCM_VCORE1 );
	CS_setDCOCenteredFrequency( CS_DCO_FREQUENCY_48 );
	CS_initClockSignal( CS_HSMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1 );
	CS_initClockSignal( CS_SMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1 );
	CS_initClockSignal( CS_MCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1 );
	CS_initClockSignal( CS_ACLK, CS_REFOCLK_SELECT, CS_CLOCK_DIVIDER_1 );
}
/*-----------------------------------------------------------*/

#if( configCREATE_SIMPLE_TICKLESS_DEMO == 0 )

	void vApplicationTickHook( void )
	{
		/* Cada interrupcin de tilde invocar esta funcin si configUSE_TICK_HOOK est configurado en 1 en
		 * FreeRTOSConfig.h. El cdigo de usuario puede agregarse aqu, pero se llama al tick hook desde un
		 * contexto de interrupcin, por lo que el cdigo no debe intentar bloquearse, y solo se pueden usar
		 * las funciones FreeRTOS API de interrupcin segura (aquellas que terminan en FromISR ()). */

		/* La demostracin completa incluye una demostracin / prueba del temporizador de software que
		 * requiere un pinchado peridico de la interrupcin de marcacin. */
		vTimerPeriodicISRTests();

		/* Llame al grupo de eventos peridicos desde la demostracin de ISR. */
		vPeriodicEventGroupsProcessing();

		/* Usa notificaciones de tareas desde una interrupcin. */
		xNotifyTaskFromISR();

		/* Use mutexes de interrupciones. */
		vInterruptSemaphorePeriodicTest();
	}

#endif
/*-----------------------------------------------------------*/






