/*
    FreeRTOS V9.0.0 - Copyright (C) 2016 Real Time Engineers Ltd.
    All rights reserved

    VISIT http://www.FreeRTOS.org TO ENSURE YOU ARE USING THE LATEST VERSION.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation >>>> AND MODIFIED BY <<<< the FreeRTOS exception.

    ***************************************************************************
    >>!   NOTE: The modification to the GPL is included to allow you to     !<<
    >>!   distribute a combined work that includes FreeRTOS without being   !<<
    >>!   obliged to provide the source code for proprietary components     !<<
    >>!   outside of the FreeRTOS kernel.                                   !<<
    ***************************************************************************

    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT ANY
    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
    FOR A PARTICULAR PURPOSE.  Full license text is available on the following
    link: http://www.freertos.org/a00114.html

    ***************************************************************************
     *                                                                       *
     *    FreeRTOS provides completely free yet professionally developed,    *
     *    robust, strictly quality controlled, supported, and cross          *
     *    platform software that is more than just the market leader, it     *
     *    is the industry's de facto standard.                               *
     *                                                                       *
     *    Help yourself get started quickly while simultaneously helping     *
     *    to support the FreeRTOS project by purchasing a FreeRTOS           *
     *    tutorial book, reference manual, or both:                          *
     *    http://www.FreeRTOS.org/Documentation                              *
     *                                                                       *
    ***************************************************************************

    http://www.FreeRTOS.org/FAQHelp.html - Having a problem?  Start by reading
    the FAQ page "My application does not run, what could be wrong?".  Have you
    defined configASSERT()?

    http://www.FreeRTOS.org/support - In return for receiving this top quality
    embedded software for free we request you assist our global community by
    participating in the support forum.

    http://www.FreeRTOS.org/training - Investing in training allows your team to
    be as productive as possible as early as possible.  Now you can receive
    FreeRTOS training directly from Richard Barry, CEO of Real Time Engineers
    Ltd, and the world's leading authority on the world's leading RTOS.

    http://www.FreeRTOS.org/plus - A selection of FreeRTOS ecosystem products,
    including FreeRTOS+Trace - an indispensable productivity tool, a DOS
    compatible FAT file system, and our tiny thread aware UDP/IP stack.

    http://www.FreeRTOS.org/labs - Where new FreeRTOS products go to incubate.
    Come and try FreeRTOS+TCP, our new open source TCP/IP stack for FreeRTOS.

    http://www.OpenRTOS.com - Real Time Engineers ltd. license FreeRTOS to High
    Integrity Systems ltd. to sell under the OpenRTOS brand.  Low cost OpenRTOS
    licenses offer ticketed support, indemnification and commercial middleware.

    http://www.SafeRTOS.com - High Integrity Systems also provide a safety
    engineered and independently SIL3 certified version for use in safety and
    mission critical applications that require provable dependability.

    1 tab == 4 spaces!
*/

/*
 *	CONDUCTOR DE PUERTO SERIAL IMPULSADO BSICO.
 *
 *	Nota 1: este controlador se usa especficamente para proporcionar una interfaz
 *	para el intrprete de comandos FreeRTOS + CLI. * No * pretende ser un controlador
 *	de puerto serie genrico. Tampoco pretende ser utilizado como un ejemplo de
 *	implementacin eficiente. En particular, se utiliza una cola para almacenar los
 *	caracteres recibidos, lo cual est bien en este caso ya que las pulsaciones de
 *	teclas llegan lentamente, pero se debe usar un DMA y / o memoria RAM en lugar de
 *	la cola en aplicaciones que esperan un mayor rendimiento.
 *
 *	Nota 2: este controlador no intenta controlar los errores UART.
*/

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"

/* Demo application includes. */
#include "serial.h"

/*-----------------------------------------------------------*/

/*
 * The UART interrupt handler.
 * El controlador de interrupcin UART.
 */
void vUART_Handler( void );

/*-----------------------------------------------------------*/

/* La cola en la que se reciben las pulsaciones de teclas recibidas.
 * NOTE LOS COMENTARIOS EN LA PARTE SUPERIOR DE ESTE ARCHIVO CON RESPECTO AL USO DE COLAS PARA ESTE PROPSITO. */
static QueueHandle_t xRxQueue = NULL;

/* Variables utilizadas en la interrupcin de Tx para enviar una cadena. */
static volatile const signed char *pcStringStart = NULL, *pcStringEnd = NULL;
static volatile TaskHandle_t xTransmittingTask = NULL;

static EUSCI_A_Type * const pxUARTA0 = ( EUSCI_A_Type * ) EUSCI_A0_BASE;

/* Configuracin UART para 19200 baudios. Valor generado utilizando la herramienta proporcionada
 * en la pgina siguiente:
http://software-dl.ti.com/msp430/msp430_public_sw/mcu/msp430/MSP430BaudRateConverter/index.html
 */
const eUSCI_UART_Config xUARTConfig =
{
	EUSCI_A_UART_CLOCKSOURCE_SMCLK,	/* SMCLK Clock Source. */
	156,							/* BRDIV */
	4,								/* UCxBRF */
	0,								/* UCxBRS */
	EUSCI_A_UART_NO_PARITY,			/* No Parity. */
	EUSCI_A_UART_LSB_FIRST,			/* MSB First. */
	EUSCI_A_UART_ONE_STOP_BIT,		/* One stop bit. */
	EUSCI_A_UART_MODE,				/* UART mode. */
	EUSCI_A_UART_OVERSAMPLING_BAUDRATE_GENERATION /* Low Frequency Mode. */
};

/*
 * Vea el archivo de encabezado serial2.h.
 */
xComPortHandle xSerialPortInitMinimal( unsigned long ulWantedBaud, unsigned long uxQueueLength )
{
    /* Crea la cola utilizada para mantener los caracteres recibidos. NOTE LOS COMENTARIOS
    EN LA PARTE SUPERIOR DE ESTE ARCHIVO CON RESPECTO AL USO DE COLAS PARA ESTE PURPSOE. */
	xRxQueue = xQueueCreate( uxQueueLength, sizeof( char ) );
	configASSERT( xRxQueue );

	/* Use las funciones de la biblioteca para inicializar y habilitar el UART. */
	MAP_UART_initModule( EUSCI_A0_BASE, &xUARTConfig );
	MAP_UART_enableModule( EUSCI_A0_BASE );
	MAP_UART_clearInterruptFlag( EUSCI_A0_BASE, EUSCI_A_UART_RECEIVE_INTERRUPT | EUSCI_A_UART_TRANSMIT_INTERRUPT );
	MAP_UART_enableInterrupt( EUSCI_A0_BASE, EUSCI_A_UART_RECEIVE_INTERRUPT );

	/* El controlador de interrupcin utiliza la funcin FreeRTOS API, por lo que su prioridad
	 * debe ser igual o inferior a la prioridad mxima de interrupcin de llamada del sistema
	 * configurada. configKERNEL_INTERRUPT_PRIORITY es la prioridad utilizada por el ticker de
	 * RTOS y (debera) siempre establecerse con la prioridad mnima. */
	MAP_Interrupt_setPriority( INT_EUSCIA0, configKERNEL_INTERRUPT_PRIORITY );
	MAP_Interrupt_enableInterrupt( INT_EUSCIA0 );

	/* Solo se admite un UART para que no se use el identificador. */
	return ( xComPortHandle ) 0;
}
/*-----------------------------------------------------------*/

BaseType_t xSerialGetChar( xComPortHandle pxPort, signed char *pcRxedChar, TickType_t xBlockTime )
{
BaseType_t xReturn;

	/* Solo se admite un solo puerto. */
	( void ) pxPort;

	/* Obtenga un carcter recibido de la cola - ingrese el estado Bloqueado
	 *(para no consumir ningn tiempo de procesamiento) para esperar a un personaje si uno no est disponible. */
	xReturn = xQueueReceive( xRxQueue, pcRxedChar, xBlockTime );
	return xReturn;
}
/*-----------------------------------------------------------*/

void vSerialPutString( xComPortHandle pxPort, const signed char * const pcString, unsigned short usStringLength )
{
const TickType_t xMaxWaitTime = pdMS_TO_TICKS( 20UL * ( uint32_t ) usStringLength );

	/* Solo se admite un solo puerto. */
	( void ) pxPort;

	/* Tenga en cuenta que no hay exclusin mutua en el nivel del controlador.
	 * Si hay ms de una tarea utilizando el puerto serie, se debe proporcionar
	 * la exclusin mutua donde se llama a esta funcin. */

	/* Asegrate de que las notificaciones no estn esperando. */
	( void ) ulTaskNotifyTake( pdTRUE, 0 );

	/* Recuerde qu tarea est enviando el byte. */
	xTransmittingTask = xTaskGetCurrentTaskHandle();

	/* Marque el inicio y el final de los datos que se envan. */
	pcStringStart = pcString;
	pcStringEnd = pcStringStart + usStringLength;

	/* Comience a enviar el primer byte. */
	pxUARTA0->TXBUF = ( uint_fast8_t ) *pcString;

	/* Habilite la interrupcin y espere a que se enve el byte. La interrupcin se desactivar nuevamente en el ISR. */
	MAP_UART_enableInterrupt( EUSCI_A0_BASE, EUSCI_A_UART_TRANSMIT_INTERRUPT );
	ulTaskNotifyTake( pdTRUE, xMaxWaitTime );
}
/*-----------------------------------------------------------*/

signed portBASE_TYPE xSerialPutChar( xComPortHandle pxPort, signed char cOutChar, TickType_t xBlockTime )
{
const TickType_t xMaxWaitTime = pdMS_TO_TICKS( 20UL );

	/*Solo se admite un solo puerto. */
	( void ) pxPort;

	/* Tenga en cuenta que no hay exclusin mutua en el nivel del controlador.
	 * Si hay ms de una tarea utilizando el puerto serie, se debe proporcionar
	 * la exclusin mutua donde se llama a esta funcin. */

	/* Asegrate de que las notificaciones no estn esperando. */
	( void ) ulTaskNotifyTake( pdTRUE, 0 );

	/* Recuerde qu tarea est enviando el byte. */
	xTransmittingTask = xTaskGetCurrentTaskHandle();

	/* Marque el inicio y el final de los datos enviados, en este caso solo un byte. */
	pcStringStart = &cOutChar;
	pcStringEnd = pcStringStart + sizeof( cOutChar );

	/* Start to send the byte. */
	pxUARTA0->TXBUF = ( uint_fast8_t ) cOutChar;

	/* Habilite la interrupcin y espere a que se enve el byte. La interrupcin se desactivar nuevamente en el ISR. */
	MAP_UART_enableInterrupt( EUSCI_A0_BASE, EUSCI_A_UART_TRANSMIT_INTERRUPT );
	ulTaskNotifyTake( pdTRUE, xMaxWaitTime );

	return pdPASS;
}
/*-----------------------------------------------------------*/

void vSerialClose(xComPortHandle xPort)
{
	/* No compatible como no requerido por la aplicacin de demostracin. */
	( void ) xPort;
}
/*-----------------------------------------------------------*/

void vUART_Handler( void )
{
uint8_t ucChar;
BaseType_t xHigherPriorityTaskWoken = pdFALSE;
uint_fast8_t xInterruptStatus;

	xInterruptStatus = MAP_UART_getEnabledInterruptStatus( EUSCI_A0_BASE );

	if( ( xInterruptStatus & EUSCI_A_UART_RECEIVE_INTERRUPT_FLAG ) != 0x00 )
	{
		/* Obtener el carcter. */
		ucChar = MAP_UART_receiveData( EUSCI_A0_BASE );

		/* Enva el caracter a la cola. Tenga en cuenta los comentarios en la parte superior
		 * de este archivo con respecto a la ineficiencia de este mtodo para cualquier cosa
		 * que no sean comunicaciones de ancho de banda muy bajo.
         *
		 * Si al escribir en la cola desbloquea una tarea, y la tarea desbloqueada tiene una prioridad
		 * sobre la tarea en ejecucin (la tarea que interrumpi esta interrupcin), xHigherPriorityTaskWoken
		 * se establecer en pdTRUE dentro de la funcin xQueueSendFromISR (). xHigherPriorityTaskWoken se
		 * pasa luego a portYIELD_FROM_ISR () al final de este manejador de interrupciones para solicitar un
		 * cambio de contexto para que la interrupcin regrese directamente a la tarea desbloqueada (prioridad ms alta). */
		xQueueSendFromISR( xRxQueue, &ucChar, &xHigherPriorityTaskWoken );
	}

	if( ( xInterruptStatus & EUSCI_A_UART_TRANSMIT_INTERRUPT_FLAG ) != 0x00 )
	{
		/* Hay ms caracteres para transmitir? */
		pcStringStart++;
		if( ( uint32_t ) pcStringStart < ( uint32_t ) pcStringEnd )
		{
			/* Esta es probablemente una funcin de espera bastante pesada solo para escribir en el registro de Tx.
			 * Un diseo optimizado probablemente reemplazara esto con una simple escritura de registro. */
			pxUARTA0->TXBUF = ( uint_fast8_t ) *pcStringStart;
		}
		else
		{
			/* No ms caracteres para enviar. Desactive la interrupcin y notifique la tarea, si la tarea est esperando. */
			MAP_UART_disableInterrupt( EUSCI_A0_BASE, EUSCI_A_UART_TRANSMIT_INTERRUPT );
			if( xTransmittingTask != NULL )
			{
				vTaskNotifyGiveFromISR( xTransmittingTask, &xHigherPriorityTaskWoken );
				xTransmittingTask = NULL;
			}
		}
	}


	/* portYIELD_FROM_ISR () solicitar un cambio de contexto si la ejecucin de este controlador
	 * de interrupcin caus que una tarea abandone el estado bloqueado, y la tarea que dej el
	 * estado bloqueado tiene una prioridad mayor que la tarea que se est ejecutando actualmente
	 * (la tarea interrumpida por esta interrupcin). Vea el comentario sobre las llamadas a
	 * xSemaphoreGiveFromISR () y xQueueSendFromISR () dentro de esta funcin. */
	portYIELD_FROM_ISR( xHigherPriorityTaskWoken );
}





