/*
    FreeRTOS V9.0.0 - Copyright (C) 2016 Real Time Engineers Ltd.
    All rights reserved

    VISIT http://www.FreeRTOS.org TO ENSURE YOU ARE USING THE LATEST VERSION.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation >>>> AND MODIFIED BY <<<< the FreeRTOS exception.

    ***************************************************************************
    >>!   NOTE: The modification to the GPL is included to allow you to     !<<
    >>!   distribute a combined work that includes FreeRTOS without being   !<<
    >>!   obliged to provide the source code for proprietary components     !<<
    >>!   outside of the FreeRTOS kernel.                                   !<<
    ***************************************************************************

    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT ANY
    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
    FOR A PARTICULAR PURPOSE.  Full license text is available on the following
    link: http://www.freertos.org/a00114.html

    ***************************************************************************
     *                                                                       *
     *    FreeRTOS provides completely free yet professionally developed,    *
     *    robust, strictly quality controlled, supported, and cross          *
     *    platform software that is more than just the market leader, it     *
     *    is the industry's de facto standard.                               *
     *                                                                       *
     *    Help yourself get started quickly while simultaneously helping     *
     *    to support the FreeRTOS project by purchasing a FreeRTOS           *
     *    tutorial book, reference manual, or both:                          *
     *    http://www.FreeRTOS.org/Documentation                              *
     *                                                                       *
    ***************************************************************************

    http://www.FreeRTOS.org/FAQHelp.html - Having a problem?  Start by reading
    the FAQ page "My application does not run, what could be wrong?".  Have you
    defined configASSERT()?

    http://www.FreeRTOS.org/support - In return for receiving this top quality
    embedded software for free we request you assist our global community by
    participating in the support forum.

    http://www.FreeRTOS.org/training - Investing in training allows your team to
    be as productive as possible as early as possible.  Now you can receive
    FreeRTOS training directly from Richard Barry, CEO of Real Time Engineers
    Ltd, and the world's leading authority on the world's leading RTOS.

    http://www.FreeRTOS.org/plus - A selection of FreeRTOS ecosystem products,
    including FreeRTOS+Trace - an indispensable productivity tool, a DOS
    compatible FAT file system, and our tiny thread aware UDP/IP stack.

    http://www.FreeRTOS.org/labs - Where new FreeRTOS products go to incubate.
    Come and try FreeRTOS+TCP, our new open source TCP/IP stack for FreeRTOS.

    http://www.OpenRTOS.com - Real Time Engineers ltd. license FreeRTOS to High
    Integrity Systems ltd. to sell under the OpenRTOS brand.  Low cost OpenRTOS
    licenses offer ticketed support, indemnification and commercial middleware.

    http://www.SafeRTOS.com - High Integrity Systems also provide a safety
    engineered and independently SIL3 certified version for use in safety and
    mission critical applications that require provable dependability.

    1 tab == 4 spaces!
*/

/******************************************************************************
 * Este proyecto proporciona dos aplicaciones de demostracin. Un simple proyecto
 * de estilo blinky, y una prueba ms completa y una aplicacin de demostracin.
 * La configuracin configCREATE_SIMPLE_TICKLESS_DEMO (definida en FreeRTOSConfig.h)
 * se usa para seleccionar entre los dos. La demostracin simplemente intermitente se
 * implementa y describe en main_blinky.c. La aplicacin de prueba y demostracin ms
 * exhaustiva se implementa y describe en main_full.c.
 *
 *  La demostracin completa usa FreeRTOS + CLI para crear una interfaz de lnea de comando simple a travs de UART.
 *
 *  La demo blinky usa el modo inactivo tickless de FreeRTOS para reducir el consumo de energa.
 *  Consulte las notas en la pgina web a continuacin sobre la diferencia en el ahorro de energa
 *  que se puede lograr entre el uso de la implementacin tickless genrica (como la utilizada por la demo blinky)
 *  y una implementacin tickless que se adapta especficamente al MSP432.
 *
 *  Este archivo implementa el cdigo que no es especfico de la demostracin.
 *
 *  Consulte http://www.FreeRTOS.org/TI_MSP432_Free_RTOS_Demo.html para obtener instrucciones.
 *
 */

/* Standard includes. */
#include <stdio.h>

/* Kernel includes. */
#include "FreeRTOS.h"
#include "task.h"

/*-----------------------------------------------------------*/

/*
 * Configure el hardware listo para ejecutar esta demostracin.
 */
static void prvSetupHardware( void );

/*
 * main_blinky () se usa cuando configCREATE_SIMPLE_TICKLESS_DEMO se establece en 1.
 * main_full () se usa cuando configCREATE_SIMPLE_TICKLESS_DEMO se establece en 0.
 */

extern void main_full( void );

/*-----------------------------------------------------------*/

int main( void )
 {
	/* See http://www.FreeRTOS.org/TI_MSP432_Free_RTOS_Demo.html for instructions. */
	
	/* Prepare el hardware para ejecutar esta demostracin. */
	prvSetupHardware();

	/* La configuracin configCREATE_SIMPLE_TICKLESS_DEMO se describe en la parte superior de este archivo. */
	configTOGGLE_LED();
	GPIO_toggleOutputOnPin( GPIO_PORT_P2, GPIO_PIN0 );
	main_full();


	return 0;
}
/*-----------------------------------------------------------*/

static void prvSetupHardware( void )
{
extern void FPU_enableModule( void );
	
	/* Los relojes no estn configurados aqu, pero dentro de main_full ()
	 * y main_blinky () ya que la demostracin completa usa un reloj rpido
	 * y la demo blinky usa un reloj lento. */

	/* Detener el temporizador de vigilancia.(Stop the watchdog timer.) */
	MAP_WDT_A_holdTimer();
	
	/* Asegrese de que la FPU est habilitada.*/
	FPU_enableModule();

	/* Seleccionar P1.2 y P1.3 en modo UART y P1.0 como salida (LED) */
	MAP_GPIO_setAsPeripheralModuleFunctionInputPin( GPIO_PORT_P1, GPIO_PIN2 | GPIO_PIN3, GPIO_PRIMARY_MODULE_FUNCTION );
	MAP_GPIO_setOutputLowOnPin( GPIO_PORT_P1, GPIO_PIN0 );
	MAP_GPIO_setAsOutputPin( GPIO_PORT_P1, GPIO_PIN0 );
	MAP_GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN1);
	MAP_GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN1);
    MAP_GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN0);
    MAP_GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN0);

}
/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* vApplicationMallocFailedHook () solo se invocar si configUSE_MALLOC_FAILED_HOOK
	 * est establecido en 1 en FreeRTOSConfig.h. Es una funcin de enlace que se llamar
	 * si falla una llamada a pvPortMalloc (). pvPortMalloc () es llamado internamente por
	 * el ncleo cada vez que se crea una tarea, cola, temporizador o semforo. Tambin lo
	 * llaman varias partes de la aplicacin de demostracin. Si se usan heap_1.c o heap_2.c,
	 * el tamao del montn disponible para pvPortMalloc () se define mediante configTOTAL_HEAP_SIZE
	 * en FreeRTOSConfig.h, y la funcin de la API xPortGetFreeHeapSize () se puede usar para
	 * consultar el tamao del espacio libre en el montn que permanece
	 * (aunque no proporciona informacin sobre cmo el montn restante podra estar fragmentado). */
	taskDISABLE_INTERRUPTS();
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* vApplicationIdleHook () solo se invocar si configUSE_IDLE_HOOK est establecido en 1 en
	 * FreeRTOSConfig.h. Se llamar en cada iteracin de la tarea inactiva. Es esencial que el
	 * cdigo agregado a esta funcin de enlace nunca intente bloquear de ninguna manera
	 * (por ejemplo, llame a xQueueReceive () con un tiempo de bloque especificado, o llame
	 * a vTaskDelay ()). Si la aplicacin hace uso de la funcin API vTaskDelete ()
	 * (como lo hace esta aplicacin de demostracin), tambin es importante que vApplicationIdleHook ()
	 * pueda volver a su funcin de llamada, ya que es responsabilidad de la tarea inactiva limpiar
	 * la memoria asignado por el ncleo a cualquier tarea que se haya eliminado desde entonces. */
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( TaskHandle_t pxTask, char *pcTaskName )
{
	( void ) pcTaskName;
	( void ) pxTask;

	/* La comprobacin de desbordamiento de la pila de tiempo de ejecucin
	 * se realiza si configCHECK_FOR_STACK_OVERFLOW est definido en 1 o 2.
	 * Esta funcin de enlace se invoca si se detecta un desbordamiento de pila. */
	taskDISABLE_INTERRUPTS();
	for( ;; );
}
/*-----------------------------------------------------------*/

void *malloc( size_t xSize )
{
	/* No debera haber un montn definido, as que intercepte cualquier intento de llamar a malloc. */
	Interrupt_disableMaster();
	for( ;; );
}
/*-----------------------------------------------------------*/

